@@ -31,13 +31,14 @@
 
 #include "base/singleton.h"
 #include "base/util.h"
-#include "converter/character_form_manager.h"
+#include "config/character_form_manager.h"
 
 namespace mozc {
 namespace composer {
-
 namespace {
 
+using ::mozc::config::CharacterFormManager;
+
 class DefaultTransliterator : public TransliteratorInterface {
  public:
   virtual ~DefaultTransliterator() {}
@@ -237,5 +238,6 @@ const TransliteratorInterface*
 TransliteratorsJa::GetHalfAsciiTransliterator() {
   return Singleton<HalfAsciiTransliterator>::get();
 }
+
 }  // namespace composer
 }  // namespace mozc
