@@ -30,7 +30,9 @@
 #ifndef MOZC_CONVERTER_NODE_ALLOCATOR_H_
 #define MOZC_CONVERTER_NODE_ALLOCATOR_H_
 
+#include "base/base.h"
 #include "base/freelist.h"
+#include "base/logging.h"
 #include "converter/node.h"
 
 namespace mozc {
