@@ -42,22 +42,24 @@
 
 #include "base/base.h"
 #include "base/flags.h"
+#include "base/logging.h"
 #include "base/mmap.h"
 #include "base/singleton.h"
+#include "base/string_piece.h"
 #include "base/trie.h"
 #include "base/util.h"
 #include "converter/node.h"
 #include "dictionary/dictionary_token.h"
 #include "dictionary/file/dictionary_file.h"
-#include "dictionary/rx/rx_trie.h"
-#include "dictionary/rx/rbx_array.h"
 #include "dictionary/system/codec_interface.h"
 #include "dictionary/system/words_info.h"
 #include "dictionary/text_dictionary_loader.h"
 #include "session/commands.pb.h"
 #include "session/request_handler.h"
 
+
 namespace mozc {
+namespace dictionary {
 namespace {
 
 // rbx_array default setting
@@ -98,16 +100,182 @@ bool IsCacheAvailable(
   return true;
 }
 
+
+// Note that this class is just introduced due to performance reason.
+// Conceptually, it should be in somewhere close to the codec implementation
+// (see comments in Next method for details).
+// However, it is necessary to refactor a bit larger area, especially around
+// codec implementations, to make it happen.
+// Considering the merit to introduce this class, we temporarily put it here.
+// TODO(hidehiko): Move this class into a Codec related file.
+class TokenDecodeIterator {
+ public:
+  typedef SystemDictionary::TrieType TrieType;
+
+  TokenDecodeIterator(
+      const SystemDictionaryCodecInterface *codec,
+      const TrieType *value_trie,
+      const uint32 *frequent_pos,
+      const string &key,
+      const uint8 *ptr)
+      : codec_(codec),
+        value_trie_(value_trie),
+        frequent_pos_(frequent_pos),
+        key_(key),
+        state_(HAS_NEXT),
+        ptr_(ptr),
+        token_info_(NULL) {
+    token_.key = key;
+    NextInternal();
+  }
+
+  ~TokenDecodeIterator() {
+  }
+
+  const TokenInfo& Get() const { return token_info_; }
+  bool Done() const { return state_ == DONE; }
+
+  void Next() {
+    DCHECK_NE(state_, DONE);
+    if (state_ == LAST_TOKEN) {
+      state_ = DONE;
+      return;
+    }
+    NextInternal();
+  }
+
+ private:
+  enum State {
+    HAS_NEXT,
+    LAST_TOKEN,
+    DONE,
+  };
+
+  void NextInternal() {
+    // Reset token_info with preserving some needed info in previous token.
+    int prev_id_in_value_trie = token_info_.id_in_value_trie;
+    token_info_.Clear();
+    token_info_.token = &token_;
+
+    // Do not clear key in token.
+    token_info_.token->attributes = Token::NONE;
+
+    // This implementation is depending on the internal behavior of DecodeToken
+    // especially which fields are updated or not. Important fields are:
+    // Token::key, Token::value : key and value are never updated.
+    // Token::cost : always updated.
+    // Token::lid, Token::rid : updated iff the pos_type is neither
+    //   FREQUENT_POS nor SAME_AS_PREV_POS.
+    // Token::attributes : updated iff the value is SPELLING_COLLECTION.
+    // TokenInfo::id_in_value_trie : updated iff the value_type is
+    //   DEFAULT_VALUE.
+    // Thus, by not-reseting Token instance intentionally, we can skip most
+    //   SAME_AS_PREV operations.
+    // The exception is Token::attributes. It is not-always set, so we need
+    // reset it everytime.
+    // This kind of structure should be packed in the codec or some
+    // related but new class.
+    int read_bytes;
+    if (!codec_->DecodeToken(ptr_, &token_info_, &read_bytes)) {
+      state_ = LAST_TOKEN;
+    }
+    ptr_ += read_bytes;
+
+    // Fill remaining values.
+    switch (token_info_.value_type) {
+      case TokenInfo::DEFAULT_VALUE: {
+        token_.value.clear();
+        LookupValue(token_info_.id_in_value_trie, &token_.value);
+        break;
+      }
+      case TokenInfo::SAME_AS_PREV_VALUE: {
+        DCHECK_NE(prev_id_in_value_trie, -1);
+        token_info_.id_in_value_trie = prev_id_in_value_trie;
+        // We can keep the current value here.
+        break;
+      }
+      case TokenInfo::AS_IS_HIRAGANA: {
+        token_.value = token_.key;
+        break;
+      }
+      case TokenInfo::AS_IS_KATAKANA: {
+        if (!key_.empty() && key_katakana_.empty()) {
+          Util::HiraganaToKatakana(key_, &key_katakana_);
+        }
+        token_.value = key_katakana_;
+        break;
+      }
+      default: {
+        LOG(DFATAL) << "unknown value_type: " << token_info_.value_type;
+        break;
+      }
+    }
+
+    if (token_info_.pos_type == TokenInfo::FREQUENT_POS) {
+      const uint32 pos = frequent_pos_[token_info_.id_in_frequent_pos_map];
+      token_.lid = pos >> 16;
+      token_.rid = pos & 0xffff;
+    }
+  }
+
+  void LookupValue(int id, string *value) const {
+    string encoded_value;
+    value_trie_->ReverseLookup(id, &encoded_value);
+    codec_->DecodeValue(encoded_value, value);
+  }
+
+  const SystemDictionaryCodecInterface *codec_;
+  const TrieType *value_trie_;
+  const uint32 *frequent_pos_;
+
+  const string &key_;
+  // Katakana key will be lazily initialized.
+  string key_katakana_;
+
+  State state_;
+  const uint8 *ptr_;
+
+  TokenInfo token_info_;
+  Token token_;
+
+  DISALLOW_COPY_AND_ASSIGN(TokenDecodeIterator);
+};
+
+
+Node *CreateNodeFromToken(
+    NodeAllocatorInterface *allocator, const Token &token, int penalty) {
+  Node *new_node = NULL;
+  // TODO(hidehiko): DCHECK nullable of allocator, and use allocator
+  // always even for unit tests.
+  if (allocator != NULL) {
+    new_node = allocator->NewNode();
+  } else {
+    // for test
+    new_node = new Node();
+  }
+  new_node->lid = token.lid;
+  new_node->rid = token.rid;
+  new_node->wcost = token.cost + penalty;
+  new_node->key = token.key;
+  new_node->value = token.value;
+  new_node->node_type = Node::NOR_NODE;
+  if (token.attributes & Token::SPELLING_CORRECTION) {
+    new_node->attributes |= Node::SPELLING_CORRECTION;
+  }
+  return new_node;
+}
+
 }  // namespace
 
 SystemDictionary::SystemDictionary()
-    : key_trie_(new rx::RxTrie),
-      value_trie_(new rx::RxTrie),
-      token_array_(new rx::RbxArray),
+    : key_trie_(new TrieType),
+      value_trie_(new TrieType),
+      token_array_(new ArrayType),
       dictionary_file_(new DictionaryFile),
       frequent_pos_(NULL),
       codec_(dictionary::SystemDictionaryCodecFactory::GetCodec()),
-      empty_limit_(Limit()) {}
+      empty_limit_(Limit()) {
+}
 
 SystemDictionary::~SystemDictionary() {}
 
@@ -182,10 +350,14 @@ bool SystemDictionary::OpenDictionaryFile() {
   const unsigned char *token_image =
       reinterpret_cast<const unsigned char *>(dictionary_file_->GetSection(
           codec_->GetSectionNameForTokens(), &len));
+#ifdef MOZC_USE_MOZC_LOUDS
+  token_array_->Open(token_image);
+#else
   if (!(token_array_->OpenImage(token_image))) {
     LOG(ERROR) << "can not open tokens array";
     return false;
   }
+#endif  // MOZC_USE_MOZC_LOUDS
 
   frequent_pos_ =
       reinterpret_cast<const uint32*>(dictionary_file_->GetSection(
@@ -203,10 +375,26 @@ Node *SystemDictionary::LookupPredictiveWithLimit(
     const char *str, int size,
     const Limit &lookup_limit,
     NodeAllocatorInterface *allocator) const {
+  if (size == 0) {
+    // To fill the gap of the behavior between rx/mozc's louds, return
+    // NULL immediately if the key is empty.
+    // Background:
+    // - For predictive search, RxTrie returns no entries because rx doesn't
+    //   invoke callbacks if the key is empty.
+    // - On the other hand, Mozc's louds trie returns all values in the trie.
+    // From the trie's point of view, returning all values looks a bit more
+    // natrual. So, we fill the gap at the dictionary layer as a short term
+    // fix.
+    // TODO(hidehiko): Returning all entries in dictionary for predictive
+    //   searching with an empty key may look natural as well. So we should
+    //   find an appropriate handling point.
+    return NULL;
+  }
+
   string lookup_key_str;
   codec_->EncodeKey(string(str, size), &lookup_key_str);
 
-  vector<rx::RxEntry> results;
+  vector<EntryType> results;
   int limit = -1;  // no limit
 
 
@@ -262,7 +450,7 @@ Node *SystemDictionary::LookupPrefixWithLimit(
   string lookup_key_str;
   codec_->EncodeKey(string(str, size), &lookup_key_str);
 
-  vector<rx::RxEntry> results;
+  vector<EntryType> results;
   int limit = -1;  // no limit
 
 
@@ -286,14 +474,31 @@ Node *SystemDictionary::LookupPrefix(
   return LookupPrefixWithLimit(str, size, empty_limit_, allocator);
 }
 
+Node *SystemDictionary::LookupExact(const char *str, int size,
+                                    NodeAllocatorInterface *allocator) const {
+  string lookup_key_str;
+  codec_->EncodeKey(string(str, size), &lookup_key_str);
+  vector<EntryType> results;
+  if (allocator != NULL) {
+    key_trie_->PrefixSearchWithLimit(
+        lookup_key_str, allocator->max_nodes_size(), &results);
+  } else {
+    key_trie_->PrefixSearch(lookup_key_str, &results);
+  }
+  FilterInfo filter;
+  filter.key_len_lower_limit = size;
+  filter.key_len_upper_limit = size;
+  int limit = -1;  // No limit
+  return GetNodesFromLookupResults(filter, results, allocator, &limit);
+}
+
 Node *SystemDictionary::GetNodesFromLookupResults(
     const FilterInfo &filter,
-    const vector<rx::RxEntry> &results,
+    const vector<EntryType> &results,
     NodeAllocatorInterface *allocator,
     int *limit) const {
   DCHECK(limit);
   Node *res = NULL;
-  vector<dictionary::TokenInfo> tokens;
   for (size_t i = 0; i < results.size(); ++i) {
     if (*limit == 0) {
       break;
@@ -326,22 +531,20 @@ Node *SystemDictionary::GetNodesFromLookupResults(
     }
 
     // gets tokens block of this key.
+#ifdef MOZC_USE_MOZC_LOUDS
+    size_t dummy_length;
+    const uint8 *encoded_tokens_ptr = reinterpret_cast<const uint8*>(
+        token_array_->Get(results[i].id, &dummy_length));
+#else
     const uint8 *encoded_tokens_ptr = token_array_->Get(results[i].id);
-    tokens.clear();
-    codec_->DecodeTokens(encoded_tokens_ptr, &tokens);
-
+#endif  // MOZC_USE_MOZC_LOUDS
     res = AppendNodesFromTokens(filter,
                                 tokens_key,
                                 actual_key,
-                                &tokens,
+                                encoded_tokens_ptr,
                                 res,
                                 allocator,
                                 limit);
-    // delete tokens
-    for (size_t j = 0; j < tokens.size(); ++j) {
-      delete tokens[j].token;
-    }
-    tokens.clear();
   }
   return res;
 }
@@ -350,101 +553,43 @@ Node *SystemDictionary::AppendNodesFromTokens(
     const FilterInfo &filter,
     const string &tokens_key,
     const string &actual_key,
-    vector<dictionary::TokenInfo> *tokens,
+    const uint8 *encoded_tokens_ptr,
     Node *node,
     NodeAllocatorInterface *allocator,
     int *limit) const {
   DCHECK(limit);
+  if (*limit == 0) {
+    return node;
+  }
 
-  string key_katakana;
-  Util::HiraganaToKatakana(tokens_key, &key_katakana);
+  int penalty = 0;
 
   Node *res = node;
-  for (size_t i = 0; i < tokens->size(); ++i) {
-    if (*limit == 0) {
-      break;
-    }
-
-    const dictionary::TokenInfo *prev_token_info =
-        ((i > 0) ? &(tokens->at(i - 1)) : NULL);
-    dictionary::TokenInfo *token_info = &(tokens->at(i));
-
-    FillTokenInfo(tokens_key, key_katakana, prev_token_info, token_info);
-
-    if (token_info->value_type == dictionary::TokenInfo::DEFAULT_VALUE) {
-      // Actual lookup here
-      LookupValue(token_info);
-    }
-
-    if (IsBadToken(filter, *token_info)) {
+  for (TokenDecodeIterator iter(
+           codec_, value_trie_.get(),
+           frequent_pos_, actual_key, encoded_tokens_ptr);
+       !iter.Done(); iter.Next()) {
+    const TokenInfo &token_info = iter.Get();
+    if (IsBadToken(filter, token_info)) {
       continue;
     }
 
+    Node *new_node = CreateNodeFromToken(
+        allocator, *token_info.token, penalty);
+    new_node->bnext = res;
+    res = new_node;
 
-    if (*limit == -1 || *limit > 0) {
-      Node *new_node = CopyTokenToNode(allocator, *(token_info->token));
-      new_node->actual_key = actual_key;
-      new_node->bnext = res;
-      res = new_node;
-      if (*limit > 0) {
-        --(*limit);
+    // *limit may be negative value, which means no-limit.
+    if (*limit > 0) {
+      --(*limit);
+      if (*limit == 0) {
+        break;
       }
     }
   }
   return res;
 }
 
-void SystemDictionary::FillTokenInfo(
-    const string &key,
-    const string &key_katakana,
-    const dictionary::TokenInfo *prev_token_info,
-    dictionary::TokenInfo *token_info) const {
-  Token *token = token_info->token;
-  token->key = key;
-
-  switch (token_info->value_type) {
-    case dictionary::TokenInfo::DEFAULT_VALUE: {
-      // Lookup value later to reduce looking up in filtered condition
-      break;
-    }
-    case dictionary::TokenInfo::SAME_AS_PREV_VALUE: {
-      DCHECK(prev_token_info != NULL);
-      token_info->id_in_value_trie = prev_token_info->id_in_value_trie;
-      token->value = prev_token_info->token->value;
-      break;
-    }
-    case dictionary::TokenInfo::AS_IS_HIRAGANA: {
-      token->value = key;
-      break;
-    }
-    case dictionary::TokenInfo::AS_IS_KATAKANA: {
-      token->value = key_katakana;
-      break;
-    }
-    default: {
-      DCHECK(!token->value.empty());
-      break;
-    }
-  }
-  switch (token_info->pos_type) {
-    case dictionary::TokenInfo::SAME_AS_PREV_POS: {
-      DCHECK(prev_token_info != NULL);
-      token->lid = prev_token_info->token->lid;
-      token->rid = prev_token_info->token->rid;
-      break;
-    }
-    case dictionary::TokenInfo::FREQUENT_POS: {
-      const uint32 pos = frequent_pos_[token_info->id_in_frequent_pos_map];
-      token->lid = pos >> 16;
-      token->rid = pos & 0xffff;
-      break;
-    }
-    default: {
-      break;
-    }
-  }
-}
-
 bool SystemDictionary::IsBadToken(
     const FilterInfo &filter,
     const dictionary::TokenInfo &token_info) const {
@@ -471,14 +616,6 @@ bool SystemDictionary::IsBadToken(
   return false;
 }
 
-void SystemDictionary::LookupValue(dictionary::TokenInfo *token_info) const {
-  const int id = token_info->id_in_value_trie;
-  string encoded_value;
-  value_trie_->ReverseLookup(id, &encoded_value);
-  string *value = &(token_info->token->value);
-  codec_->DecodeValue(encoded_value, value);
-}
-
 Node *SystemDictionary::LookupReverse(
     const char *str, int size,
     NodeAllocatorInterface *allocator) const {
@@ -516,7 +653,7 @@ void SystemDictionary::PopulateReverseLookupCache(
     const string suffix = string(&str[pos], size - pos);
     string lookup_key;
     codec_->EncodeValue(suffix, &lookup_key);
-    vector<rx::RxEntry> results;
+    vector<EntryType> results;
     value_trie_->PrefixSearch(lookup_key, &results);
     for (size_t i = 0; i < results.size(); ++i) {
       ids.insert(results[i].id);
@@ -539,7 +676,7 @@ Node *SystemDictionary::GetReverseLookupNodesForT13N(
   string lookup_key;
   codec_->EncodeKey(hiragana, &lookup_key);
 
-  vector<rx::RxEntry> results;
+  vector<EntryType> results;
   if (*limit != -1) {
     key_trie_->PrefixSearchWithLimit(lookup_key, *limit, &results);
   } else {
@@ -560,7 +697,7 @@ Node *SystemDictionary::GetReverseLookupNodesForValue(
   string lookup_key;
   codec_->EncodeValue(value, &lookup_key);
 
-  vector<rx::RxEntry> value_results;
+  vector<EntryType> value_results;
 
   if (*limit != -1) {
     value_trie_->PrefixSearchWithLimit(lookup_key, *limit, &value_results);
@@ -597,7 +734,13 @@ void SystemDictionary::ScanTokens(
   int offset = 0;
   int tokens_offset = 0;
   int index = 0;
+#ifdef MOZC_USE_MOZC_LOUDS
+  size_t dummy_length;
+  const uint8 *encoded_tokens_ptr =
+      reinterpret_cast<const uint8*>(token_array_->Get(0, &dummy_length));
+#else
   const uint8 *encoded_tokens_ptr = token_array_->Get(0);
+#endif  // MOZC_USE_MOZC_LOUDS
   const uint8 termination_flag = codec_->GetTokensTerminationFlag();
   while (encoded_tokens_ptr[offset] != termination_flag) {
     int read_bytes;
@@ -632,8 +775,13 @@ Node *SystemDictionary::GetNodesFromReverseLookupResults(
     NodeAllocatorInterface *allocator,
     int *limit) const {
   Node *res = NULL;
-  vector<dictionary::TokenInfo> tokens;
+#ifdef MOZC_USE_MOZC_LOUDS
+  size_t dummy_length;
+  const uint8 *encoded_tokens_ptr =
+      reinterpret_cast<const uint8*>(token_array_->Get(0, &dummy_length));
+#else
   const uint8 *encoded_tokens_ptr = token_array_->Get(0);
+#endif  // MOZC_USE_MOZC_LOUDS
   for (set<int>::const_iterator set_itr = id_set.begin();
        set_itr != id_set.end();
        ++set_itr) {
@@ -653,53 +801,24 @@ Node *SystemDictionary::GetNodesFromReverseLookupResults(
 
       const ReverseLookupResult &reverse_result = result_itr->second;
 
-      tokens.clear();
-      codec_->DecodeTokens(
-          encoded_tokens_ptr + reverse_result.tokens_offset, &tokens);
-
       string encoded_key;
       key_trie_->ReverseLookup(reverse_result.id_in_key_trie, &encoded_key);
       string tokens_key;
       codec_->DecodeKey(encoded_key, &tokens_key);
-      // always the same for reverse conversions.
-      string actual_key = tokens_key;
 
-      res = AppendNodesFromTokens(filter,
-                                  tokens_key,
-                                  actual_key,
-                                  &tokens,
-                                  res,
-                                  allocator,
-                                  limit);
-
-      // delete tokens
-      for (size_t i = 0; i < tokens.size(); ++i) {
-        delete tokens[i].token;
-      }
-      tokens.clear();
+      // actual_key is always the same as tokens_key for reverse conversions.
+      res = AppendNodesFromTokens(
+          filter,
+          tokens_key,
+          tokens_key,
+          encoded_tokens_ptr + reverse_result.tokens_offset,
+          res,
+          allocator,
+          limit);
     }
   }
   return res;
 }
 
-Node *SystemDictionary::CopyTokenToNode(NodeAllocatorInterface *allocator,
-                                        const Token &token) const {
-  Node *new_node = NULL;
-  if (allocator != NULL) {
-    new_node = allocator->NewNode();
-  } else {
-    // for test
-    new_node = new Node();
-  }
-  new_node->lid = token.lid;
-  new_node->rid = token.rid;
-  new_node->wcost = token.cost;
-  new_node->key = token.key;
-  new_node->value = token.value;
-  new_node->node_type = Node::NOR_NODE;
-  if (token.attributes & Token::SPELLING_CORRECTION) {
-    new_node->attributes |= Node::SPELLING_CORRECTION;
-  }
-  return new_node;
-}
+}  // namespace dictionary
 }  // namespace mozc
