@@ -37,18 +37,25 @@
 #include "base/base.h"
 #include "base/file_stream.h"
 #include "base/hash_tables.h"
+#include "base/logging.h"
 #include "base/util.h"
 #include "data_manager/user_pos_manager.h"
 #include "dictionary/dictionary_token.h"
 #include "dictionary/file/codec_interface.h"
 #include "dictionary/pos_matcher.h"
-#include "dictionary/rx/rbx_array_builder.h"
-#include "dictionary/rx/rx_trie_builder.h"
 #include "dictionary/system/codec.h"
 #include "dictionary/system/codec_interface.h"
 #include "dictionary/system/words_info.h"
 #include "dictionary/text_dictionary_loader.h"
 
+#ifdef MOZC_USE_MOZC_LOUDS
+#include "storage/louds/bit_vector_based_array_builder.h"
+#include "storage/louds/louds_trie_builder.h"
+#else
+#include "dictionary/rx/rbx_array_builder.h"
+#include "dictionary/rx/rx_trie_builder.h"
+#endif  // MOZC_USE_MOZC_LOUDS
+
 DEFINE_bool(preserve_intermediate_dictionary, false,
             "preserve inetemediate dictionary file.");
 DEFINE_int32(min_key_length_to_use_small_cost_encoding, 6,
@@ -81,9 +88,9 @@ struct TokenGreaterThan {
 };
 
 SystemDictionaryBuilder::SystemDictionaryBuilder()
-    : value_trie_builder_(new rx::RxTrieBuilder),
-      key_trie_builder_(new rx::RxTrieBuilder),
-      token_array_builder_(new rx::RbxArrayBuilder),
+    : value_trie_builder_(new TrieBuilder),
+      key_trie_builder_(new TrieBuilder),
+      token_array_builder_(new ArrayBuilder),
       codec_(SystemDictionaryCodecFactory::GetCodec()) {}
 
 SystemDictionaryBuilder::~SystemDictionaryBuilder() {}
@@ -103,21 +110,21 @@ void SystemDictionaryBuilder::BuildFromFile(const string &input_file) {
 }
 
 void SystemDictionaryBuilder::BuildFromTokens(const vector<Token *> &tokens) {
-  KeyInfoMap key_info_map;
-  ReadTokens(tokens, &key_info_map);
+  KeyInfoList key_info_list;
+  ReadTokens(tokens, &key_info_list);
 
-  BuildFrequentPos(key_info_map);
-  BuildValueTrie(key_info_map);
-  BuildKeyTrie(key_info_map);
+  BuildFrequentPos(key_info_list);
+  BuildValueTrie(key_info_list);
+  BuildKeyTrie(key_info_list);
 
-  SetIdForValue(&key_info_map);
-  SetIdForKey(&key_info_map);
-  SortTokenInfo(&key_info_map);
-  SetCostType(&key_info_map);
-  SetPosType(&key_info_map);
-  SetValueType(&key_info_map);
+  SetIdForValue(&key_info_list);
+  SetIdForKey(&key_info_list);
+  SortTokenInfo(&key_info_list);
+  SetCostType(&key_info_list);
+  SetPosType(&key_info_list);
+  SetValueType(&key_info_list);
 
-  BuildTokenArray(key_info_map);
+  BuildTokenArray(key_info_list);
 }
 
 void SystemDictionaryBuilder::WriteToFile(const string &output_file) const {
@@ -132,34 +139,54 @@ void SystemDictionaryBuilder::WriteToStream(
   vector<DictionaryFileSection> sections;
   DictionaryFileCodecInterface *file_codec =
       DictionaryFileCodecFactory::GetCodec();
-  DictionaryFileSection value_trie_section = {
+#ifdef MOZC_USE_MOZC_LOUDS
+  DictionaryFileSection value_trie_section(
+    value_trie_builder_->image().data(),
+    value_trie_builder_->image().size(),
+    file_codec->GetSectionName(codec_->GetSectionNameForValue()));
+#else
+  DictionaryFileSection value_trie_section(
     value_trie_builder_->GetImageBody(),
     value_trie_builder_->GetImageSize(),
-    file_codec->GetSectionName(codec_->GetSectionNameForValue()),
-  };
+    file_codec->GetSectionName(codec_->GetSectionNameForValue()));
+#endif  // MOZC_USE_MOZC_LOUDS
   sections.push_back(value_trie_section);
-  DictionaryFileSection key_trie_section = {
+
+#ifdef MOZC_USE_MOZC_LOUDS
+  DictionaryFileSection key_trie_section(
+    key_trie_builder_->image().data(),
+    key_trie_builder_->image().size(),
+    file_codec->GetSectionName(codec_->GetSectionNameForKey()));
+#else
+  DictionaryFileSection key_trie_section(
     key_trie_builder_->GetImageBody(),
     key_trie_builder_->GetImageSize(),
-    file_codec->GetSectionName(codec_->GetSectionNameForKey()),
-  };
+    file_codec->GetSectionName(codec_->GetSectionNameForKey()));
+#endif  // MOZC_USE_MOZC_LOUDS
   sections.push_back(key_trie_section);
-  DictionaryFileSection token_array_section = {
+
+#ifdef MOZC_USE_MOZC_LOUDS
+  DictionaryFileSection token_array_section(
+    token_array_builder_->image().data(),
+    token_array_builder_->image().size(),
+    file_codec->GetSectionName(codec_->GetSectionNameForTokens()));
+#else
+  DictionaryFileSection token_array_section(
     token_array_builder_->GetImageBody(),
     token_array_builder_->GetImageSize(),
-    file_codec->GetSectionName(codec_->GetSectionNameForTokens()),
-  };
+    file_codec->GetSectionName(codec_->GetSectionNameForTokens()));
+#endif  // MOZC_USE_MOZC_LOUDS
+
   sections.push_back(token_array_section);
   uint32 frequent_pos_array[256] = {0};
   for (map<uint32, int>::const_iterator i = frequent_pos_.begin();
        i != frequent_pos_.end(); ++i) {
     frequent_pos_array[i->second] = i->first;
   }
-  DictionaryFileSection frequent_pos_section = {
+  DictionaryFileSection frequent_pos_section(
     reinterpret_cast<const char *>(frequent_pos_array),
     sizeof frequent_pos_array,
-    file_codec->GetSectionName(codec_->GetSectionNameForPos()),
-  };
+    file_codec->GetSectionName(codec_->GetSectionNameForPos()));
   sections.push_back(frequent_pos_section);
 
   if (FLAGS_preserve_intermediate_dictionary &&
@@ -196,49 +223,84 @@ TokenInfo::ValueType GetValueType(const Token *token) {
   return TokenInfo::DEFAULT_VALUE;
 }
 
-bool HaveHomonymsInSamePos(const KeyInfo &key_info) {
+bool HasHomonymsInSamePos(
+    const SystemDictionaryBuilder::KeyInfo &key_info) {
+  // Early exit path mainly for performance.
+  if (key_info.tokens.size() == 1) {
+    return false;
+  }
+
   hash_set<uint32> seen;
   for (size_t i = 0; i < key_info.tokens.size(); ++i) {
     const Token *token = key_info.tokens[i].token;
     const uint32 pos = GetCombinedPos(token->lid, token->rid);
-    if (seen.find(pos) == seen.end()) {
-      seen.insert(pos);
-    } else {
+    if (!seen.insert(pos).second) {
+      // Insertion failed, which means we already have |pos|.
       return true;
     }
   }
   return false;
 }
+
+struct TokenPtrLessThan {
+  inline bool operator()(const Token* lhs, const Token* rhs) const {
+    return lhs->key < rhs->key;
+  }
+};
+
 }  // namespace
 
 void SystemDictionaryBuilder::ReadTokens(const vector<Token *> &tokens,
-                                         KeyInfoMap *key_info_map) const {
+                                         KeyInfoList *key_info_list) const {
+  // Create KeyInfoList in two steps.
+  // 1. Create an array of Token with (stably) sorting Token::key.
+  //    [Token 1(key:aaa)][Token 2(key:aaa)][Token 3(key:abc)][...]
+  // 2. Group Token(s) by Token::Key and convert them into KeyInfo.
+  //    [KeyInfo(key:aaa)[Token 1][Token 2]][KeyInfo(key:abc)[Token 3]][...]
+
+  // Step 1.
+  typedef vector<Token *> ReduceBuffer;
+  ReduceBuffer reduce_buffer;
+  reduce_buffer.reserve(tokens.size());
   for (vector<Token *>::const_iterator iter = tokens.begin();
        iter != tokens.end(); ++iter) {
     Token *token = *iter;
     CHECK(!token->key.empty()) << "empty key string in input";
     CHECK(!token->value.empty()) << "empty value string in input";
-    KeyInfoMap::iterator i = key_info_map->find(token->key);
-    if (i == key_info_map->end()) {
-      pair<KeyInfoMap::iterator, bool> result =
-          key_info_map->insert(make_pair(token->key, KeyInfo()));
-      CHECK(result.second);
-      i = result.first;
-    }
-    i->second.tokens.push_back(TokenInfo(token));
-    i->second.tokens.back().value_type = GetValueType(token);
+    reduce_buffer.push_back(token);
   }
+  stable_sort(reduce_buffer.begin(), reduce_buffer.end(), TokenPtrLessThan());
+
+  // Step 2.
+  key_info_list->clear();
+  if (reduce_buffer.size() == 0) {
+    return;
+  }
+  KeyInfo last_key_info;
+  last_key_info.key = reduce_buffer.front()->key;
+  for (ReduceBuffer::const_iterator iter = reduce_buffer.begin();
+       iter != reduce_buffer.end(); ++iter) {
+    Token *token = *iter;
+    if (last_key_info.key != token->key) {
+      key_info_list->push_back(last_key_info);
+      last_key_info = KeyInfo();
+      last_key_info.key = token->key;
+    }
+    last_key_info.tokens.push_back(TokenInfo(token));
+    last_key_info.tokens.back().value_type = GetValueType(token);
+  }
+  key_info_list->push_back(last_key_info);
 }
 
 void SystemDictionaryBuilder::BuildFrequentPos(
-    const KeyInfoMap &key_info_map) {
+    const KeyInfoList &key_info_list) {
   // Calculate frequency of each pos
   // TODO(toshiyuki): It might be better to count frequency
   // with considering same_as_prev_pos.
   map<uint32, int> pos_map;
-  for (KeyInfoMap::const_iterator itr = key_info_map.begin();
-       itr != key_info_map.end(); ++itr) {
-    const KeyInfo &key_info = itr->second;
+  for (KeyInfoList::const_iterator itr = key_info_list.begin();
+       itr != key_info_list.end(); ++itr) {
+    const KeyInfo &key_info = *itr;
     for (size_t i = 0; i < key_info.tokens.size(); ++i) {
       const Token *token = key_info.tokens[i].token;
       pos_map[GetCombinedPos(token->lid, token->rid)]++;
@@ -283,10 +345,10 @@ void SystemDictionaryBuilder::BuildFrequentPos(
 }
 
 
-void SystemDictionaryBuilder::BuildValueTrie(const KeyInfoMap &key_info_map) {
-  for (KeyInfoMap::const_iterator itr = key_info_map.begin();
-       itr != key_info_map.end(); ++itr) {
-    const KeyInfo &key_info = itr->second;
+void SystemDictionaryBuilder::BuildValueTrie(const KeyInfoList &key_info_list) {
+  for (KeyInfoList::const_iterator itr = key_info_list.begin();
+       itr != key_info_list.end(); ++itr) {
+    const KeyInfo &key_info = *itr;
     for (size_t i = 0; i < key_info.tokens.size(); ++i) {
       const TokenInfo &token_info = key_info.tokens[i];
       if (token_info.value_type == TokenInfo::AS_IS_HIRAGANA ||
@@ -296,38 +358,47 @@ void SystemDictionaryBuilder::BuildValueTrie(const KeyInfoMap &key_info_map) {
       }
       string value_str;
       codec_->EncodeValue(token_info.token->value, &value_str);
+#ifdef MOZC_USE_MOZC_LOUDS
+      value_trie_builder_->Add(value_str);
+#else
       value_trie_builder_->AddKey(value_str);
+#endif  // MOZC_USE_MOZC_LOUDS
     }
   }
   value_trie_builder_->Build();
 }
 
-void SystemDictionaryBuilder::SetIdForValue(KeyInfoMap *key_info_map) const {
-  for (KeyInfoMap::iterator itr = key_info_map->begin();
-       itr != key_info_map->end(); ++itr) {
-    for (size_t i = 0; i < itr->second.tokens.size(); ++i) {
-      TokenInfo *token_info = &(itr->second.tokens[i]);
+void SystemDictionaryBuilder::SetIdForValue(KeyInfoList *key_info_list) const {
+  for (KeyInfoList::iterator itr = key_info_list->begin();
+       itr != key_info_list->end(); ++itr) {
+    for (size_t i = 0; i < itr->tokens.size(); ++i) {
+      TokenInfo *token_info = &(itr->tokens[i]);
       string value_str;
       codec_->EncodeValue(token_info->token->value, &value_str);
+#ifdef MOZC_USE_MOZC_LOUDS
+      token_info->id_in_value_trie =
+          value_trie_builder_->GetId(value_str);
+#else
       token_info->id_in_value_trie =
           value_trie_builder_->GetIdFromKey(value_str);
+#endif  // MOZC_USE_MOZC_LOUDS
     }
   }
 }
 
-void SystemDictionaryBuilder::SortTokenInfo(KeyInfoMap *key_info_map) const {
-  for (KeyInfoMap::iterator itr = key_info_map->begin();
-       itr != key_info_map->end(); ++itr) {
-    KeyInfo *key_info = &(itr->second);
+void SystemDictionaryBuilder::SortTokenInfo(KeyInfoList *key_info_list) const {
+  for (KeyInfoList::iterator itr = key_info_list->begin();
+       itr != key_info_list->end(); ++itr) {
+    KeyInfo *key_info = &(*itr);
     sort(key_info->tokens.begin(), key_info->tokens.end(), TokenGreaterThan());
   }
 }
 
-void SystemDictionaryBuilder::SetCostType(KeyInfoMap *key_info_map) const {
-  for (KeyInfoMap::iterator itr = key_info_map->begin();
-       itr != key_info_map->end(); ++itr) {
-    KeyInfo *key_info = &(itr->second);
-    if (HaveHomonymsInSamePos(*key_info)) {
+void SystemDictionaryBuilder::SetCostType(KeyInfoList *key_info_list) const {
+  for (KeyInfoList::iterator itr = key_info_list->begin();
+       itr != key_info_list->end(); ++itr) {
+    KeyInfo *key_info = &(*itr);
+    if (HasHomonymsInSamePos(*key_info)) {
       continue;
     }
     for (size_t i = 0; i < key_info->tokens.size(); ++i) {
@@ -340,10 +411,10 @@ void SystemDictionaryBuilder::SetCostType(KeyInfoMap *key_info_map) const {
   }
 }
 
-void SystemDictionaryBuilder::SetPosType(KeyInfoMap *key_info_map) const {
-  for (KeyInfoMap::iterator itr = key_info_map->begin();
-       itr != key_info_map->end(); ++itr) {
-    KeyInfo *key_info = &(itr->second);
+void SystemDictionaryBuilder::SetPosType(KeyInfoList *key_info_list) const {
+  for (KeyInfoList::iterator itr = key_info_list->begin();
+       itr != key_info_list->end(); ++itr) {
+    KeyInfo *key_info = &(*itr);
     for (size_t i = 0; i < key_info->tokens.size(); ++i) {
       TokenInfo *token_info = &(key_info->tokens[i]);
       const uint32 pos = GetCombinedPos(token_info->token->lid,
@@ -366,10 +437,10 @@ void SystemDictionaryBuilder::SetPosType(KeyInfoMap *key_info_map) const {
   }
 }
 
-void SystemDictionaryBuilder::SetValueType(KeyInfoMap *key_info_map) const {
-  for (KeyInfoMap::iterator itr = key_info_map->begin();
-       itr != key_info_map->end(); ++itr) {
-    KeyInfo *key_info = &(itr->second);
+void SystemDictionaryBuilder::SetValueType(KeyInfoList *key_info_list) const {
+  for (KeyInfoList::iterator itr = key_info_list->begin();
+       itr != key_info_list->end(); ++itr) {
+    KeyInfo *key_info = &(*itr);
     for (size_t i = 1; i < key_info->tokens.size(); ++i) {
       const TokenInfo *prev_token_info = &(key_info->tokens[i - 1]);
       TokenInfo *token_info = &(key_info->tokens[i]);
@@ -382,47 +453,68 @@ void SystemDictionaryBuilder::SetValueType(KeyInfoMap *key_info_map) const {
   }
 }
 
-void SystemDictionaryBuilder::BuildKeyTrie(const KeyInfoMap &key_info_map) {
-  for (KeyInfoMap::const_iterator itr = key_info_map.begin();
-       itr != key_info_map.end(); ++itr) {
+void SystemDictionaryBuilder::BuildKeyTrie(const KeyInfoList &key_info_list) {
+  for (KeyInfoList::const_iterator itr = key_info_list.begin();
+       itr != key_info_list.end(); ++itr) {
     string key_str;
-    codec_->EncodeKey(itr->first, &key_str);
+    codec_->EncodeKey(itr->key, &key_str);
+#ifdef MOZC_USE_MOZC_LOUDS
+    key_trie_builder_->Add(key_str);
+#else
     key_trie_builder_->AddKey(key_str);
+#endif  // MOZC_USE_MOZC_LOUDS
   }
   key_trie_builder_->Build();
 }
 
-void SystemDictionaryBuilder::SetIdForKey(KeyInfoMap *key_info_map) const {
-  for (KeyInfoMap::iterator itr = key_info_map->begin();
-       itr != key_info_map->end(); ++itr) {
+void SystemDictionaryBuilder::SetIdForKey(KeyInfoList *key_info_list) const {
+  for (KeyInfoList::iterator itr = key_info_list->begin();
+       itr != key_info_list->end(); ++itr) {
+    KeyInfo *key_info = &(*itr);
     string key_str;
-    codec_->EncodeKey(itr->first, &key_str);
-    KeyInfo *key_info = &(itr->second);
+    codec_->EncodeKey(key_info->key, &key_str);
+#ifdef MOZC_USE_MOZC_LOUDS
+    key_info->id_in_key_trie =
+        key_trie_builder_->GetId(key_str);
+#else
     key_info->id_in_key_trie =
         key_trie_builder_->GetIdFromKey(key_str);
+#endif  // MOZC_USE_MOZC_LOUDS
   }
 }
 
-void SystemDictionaryBuilder::BuildTokenArray(const KeyInfoMap &key_info_map) {
-  // map from trie id to key string.
-  vector<string> id_map;
-  id_map.resize(key_info_map.size());
-  for (KeyInfoMap::const_iterator itr = key_info_map.begin();
-       itr != key_info_map.end(); ++itr) {
-    const KeyInfo &key_info = itr->second;
-    const int id = key_info.id_in_key_trie;
-    id_map[id] = itr->first;
+void SystemDictionaryBuilder::BuildTokenArray(
+    const KeyInfoList &key_info_list) {
+  // Here we make a reverse lookup table as follows:
+  //   |key_info_list[X].id_in_key_trie| -> |key_info_list[X]|
+  // assuming |key_info_list[X].id_in_key_trie| is unique and successive.
+  {
+    vector<const KeyInfo *> id_to_keyinfo_table;
+    id_to_keyinfo_table.resize(key_info_list.size());
+    for (KeyInfoList::const_iterator itr = key_info_list.begin();
+         itr != key_info_list.end(); ++itr) {
+      const KeyInfo &key_info = *itr;
+      const int id = key_info.id_in_key_trie;
+      id_to_keyinfo_table[id] = &key_info;
+    }
+
+    for (size_t i = 0; i < id_to_keyinfo_table.size(); ++i) {
+      const KeyInfo &key_info = *id_to_keyinfo_table[i];
+      string tokens_str;
+      codec_->EncodeTokens(key_info.tokens, &tokens_str);
+#ifdef MOZC_USE_MOZC_LOUDS
+      token_array_builder_->Add(tokens_str);
+#else
+      token_array_builder_->Push(tokens_str);
+#endif  // MOZC_USE_MOZC_LOUDS
+    }
   }
 
-  for (size_t i = 0; i < id_map.size(); ++i) {
-    KeyInfoMap::const_iterator itr = key_info_map.find(id_map[i]);
-    CHECK(itr != key_info_map.end());
-    const KeyInfo &key_info = itr->second;
-    string tokens_str;
-    codec_->EncodeTokens(key_info.tokens, &tokens_str);
-    token_array_builder_->Push(tokens_str);
-  }
+#ifdef MOZC_USE_MOZC_LOUDS
+  token_array_builder_->Add(string(1, codec_->GetTokensTerminationFlag()));
+#else
   token_array_builder_->Push(string(1, codec_->GetTokensTerminationFlag()));
+#endif
   token_array_builder_->Build();
 }
 
