@@ -31,6 +31,9 @@
 
 #include <algorithm>
 #include <map>
+
+#include "base/base.h"
+#include "base/logging.h"
 #include "base/util.h"
 
 namespace mozc {
