@@ -27,8 +27,6 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-#include <ibus.h>
-
 #include "unix/ibus/mozc_engine_property.h"
 
 #include "base/base.h"
@@ -37,8 +35,9 @@
 namespace mozc {
 namespace ibus {
 
+namespace {
 // The list of properties used in ibus-mozc.
-const MozcEngineProperty kMozcEngineProperties[] = {
+const MozcEngineProperty kMozcEnginePropertiesArray[] = {
   {
     commands::HALF_ASCII,
     "CompositionMode.English",
@@ -63,43 +62,22 @@ const MozcEngineProperty kMozcEngineProperties[] = {
     "alpha_full.png",
   },
 };
+}  // namespace
 
+const MozcEngineProperty *kMozcEngineProperties =
+    &kMozcEnginePropertiesArray[0];
 // The IMEOff state is not available in Chewing.
 const MozcEngineProperty *kMozcEnginePropertyIMEOffState = NULL;
-const size_t kMozcEnginePropertiesSize = arraysize(kMozcEngineProperties);
+const size_t kMozcEnginePropertiesSize = arraysize(kMozcEnginePropertiesArray);
 
 const commands::CompositionMode kMozcEngineInitialCompositionMode =
     commands::HIRAGANA;
 
-const MozcEngineSwitchProperty kMozcEngineSwitchProperties[] = {};
-const size_t kMozcEngineSwitchPropertiesSize =
-    ARRAYSIZE(kMozcEngineSwitchProperties);
+const MozcEngineSwitchProperty *kMozcEngineSwitchProperties = NULL;
+const size_t kMozcEngineSwitchPropertiesSize = 0;
 
-// TODO(mukai): Fix the following properties when we suppport
-// non-ChromeOS Linux.
-const MozcEngineToolProperty kMozcEngineToolProperties[] = {
-  {
-    "Tool.ConfigDialog",
-    "config_dialog",
-    "Properties",
-    "properties.png",
-  },
-  {
-    "Tool.DictionaryTool",
-    "dictionary_tool",
-    "Dictionary tool",
-    "dictionary.png",
-  },
-  {
-    "Tool.AboutDialog",
-    "about_dialog",
-    "About Mozc",
-    NULL,
-  },
-};
-
-const size_t kMozcEngineToolPropertiesSize =
-    arraysize(kMozcEngineToolProperties);
+const MozcEngineToolProperty *kMozcEngineToolProperties = NULL;
+const size_t kMozcEngineToolPropertiesSize = 0;
 
 const unsigned int kPageSize = 10;
 }  // namespace ibus
