@@ -73,8 +73,6 @@ class EnglishContext : public PinyinContextInterface {
 
   bool SelectCandidate(size_t index);
   bool FocusCandidate(size_t index);
-  bool FocusCandidatePrev();
-  bool FocusCandidateNext();
   bool ClearCandidateFromHistory(size_t index);
 
   bool RemoveCharBefore();
