@@ -74,8 +74,6 @@ class PunctuationContext : public PinyinContextInterface {
 
   virtual bool SelectCandidate(size_t index);
   virtual bool FocusCandidate(size_t index);
-  virtual bool FocusCandidatePrev();
-  virtual bool FocusCandidateNext();
   virtual bool ClearCandidateFromHistory(size_t index);
 
   virtual bool RemoveCharBefore();
