@@ -31,7 +31,10 @@
 
 #include <sstream>
 #include <string>
+#include <vector>
+
 #include "base/base.h"
+#include "base/logging.h"
 #include "base/util.h"
 
 namespace mozc {
