@@ -29,16 +29,20 @@
 
 #include "prediction/dictionary_predictor.h"
 
-#include <limits.h>   // INT_MAX
+#include <algorithm>
 #include <cctype>
+#include <climits>   // INT_MAX
 #include <cmath>
+#include <list>
 #include <map>
 #include <set>
 #include <string>
 #include <utility>
 #include <vector>
-#include <algorithm>
+
 #include "base/base.h"
+#include "base/logging.h"
+#include "base/number_util.h"
 #include "base/trie.h"
 #include "base/util.h"
 #include "composer/composer.h"
@@ -51,6 +55,7 @@
 #include "converter/node_allocator.h"
 #include "converter/segmenter_interface.h"
 #include "converter/segments.h"
+#include "data_manager/data_manager_interface.h"
 #include "dictionary/dictionary_interface.h"
 #include "dictionary/pos_matcher.h"
 #include "prediction/predictor_interface.h"
@@ -73,15 +78,6 @@ namespace {
 const size_t kSuggestionMaxNodesSize = 256;
 const size_t kPredictionMaxNodesSize = 100000;
 
-bool IsNumber(const string &str) {
-  for (string::const_iterator it = str.begin(); it != str.end(); ++it) {
-    if (!isdigit(*it)) {
-      return false;
-    }
-  }
-  return true;
-}
-
 void GetNumberSuffixArray(const string &history_input,
                           vector<string> *suffixes) {
   DCHECK(suffixes);
@@ -108,24 +104,51 @@ void GetNumberSuffixArray(const string &history_input,
   for (int j = 1; ZeroQueryNum[default_num][j]; ++j) {
     suffixes->push_back(ZeroQueryNum[default_num][j]);
   }
-  DCHECK_GE(suffixes->size(), 0);
 }
+
+// Returns true if the |target| may be reduncant node.
+bool MaybeRedundant(const Node *reference, const Node *target) {
+  return Util::StartsWith(target->value, reference->value);
+}
+
+// Comparator for sorting prediction candidates.
+// If we have words A and AB, for example "六本木" and "六本木ヒルズ",
+// assume that cost(A) < cost(AB).
+class NodeLess {
+ public:
+  bool operator() (const Node *lhs, const Node *rhs) {
+    return lhs->wcost < rhs->wcost;
+  }
+};
+
+bool IsLatinInputMode(const ConversionRequest &request) {
+  return (request.has_composer() &&
+          (request.composer().GetInputMode() == transliteration::HALF_ASCII ||
+           request.composer().GetInputMode() == transliteration::FULL_ASCII));
+}
+
 }  // namespace
 
 DictionaryPredictor::DictionaryPredictor(
     const ImmutableConverterInterface *immutable_converter,
     const DictionaryInterface *dictionary,
     const DictionaryInterface *suffix_dictionary,
-    const ConnectorInterface *connector,
-    const SegmenterInterface *segmenter,
-    const POSMatcher &pos_matcher)
+    const DataManagerInterface *data_manager)
     : immutable_converter_(immutable_converter),
       dictionary_(dictionary),
       suffix_dictionary_(suffix_dictionary),
-      connector_(connector),
-      segmenter_(segmenter),
-      counter_suffix_word_id_(pos_matcher.GetCounterSuffixWordId()),
-      predictor_name_("DictionaryPredictor") {}
+      connector_(data_manager->GetConnector()),
+      segmenter_(data_manager->GetSegmenter()),
+      counter_suffix_word_id_(
+          data_manager->GetPOSMatcher()->GetCounterSuffixWordId()),
+      predictor_name_("DictionaryPredictor") {
+  // Create suggestion filter.
+  const char *data = NULL;
+  size_t size = 0;
+  data_manager->GetSuggestionFilterData(&data, &size);
+  CHECK(data);
+  suggestion_filter_.reset(new SuggestionFilter(data, size));
+}
 
 DictionaryPredictor::~DictionaryPredictor() {}
 
@@ -149,7 +172,7 @@ bool DictionaryPredictor::PredictForRequest(const ConversionRequest &request,
 
   SetCost(*segments, &results);
   RemovePrediction(*segments, &results);
-  return AddPredictionToCandidates(segments, &results);
+  return AddPredictionToCandidates(request, segments, &results);
 }
 
 bool DictionaryPredictor::AggregatePrediction(
@@ -159,26 +182,29 @@ bool DictionaryPredictor::AggregatePrediction(
   DCHECK(segments);
   DCHECK(results);
 
-  const PredictionType prediction_type = GetPredictionType(*segments);
-  if (prediction_type == NO_PREDICTION) {
+  const PredictionTypes prediction_types =
+      GetPredictionTypes(request, *segments);
+  if (prediction_types == NO_PREDICTION) {
     return false;
   }
 
   if (segments->request_type() == Segments::PARTIAL_SUGGESTION ||
       segments->request_type() == Segments::PARTIAL_PREDICTION) {
-      // This request type is used to get conversion before cursor during
+    // This request type is used to get conversion before cursor during
     // composition mode. Thus it should return only the candidates whose key
     // exactly matches the query.
     // Therefore, we use only the realtime conversion result.
-    AggregateRealtimeConversion(prediction_type, segments, allocator, results);
+    AggregateRealtimeConversion(prediction_types, segments, allocator, results);
   } else {
-    AggregateRealtimeConversion(prediction_type, segments, allocator, results);
-    AggregateUnigramPrediction(prediction_type, request, segments,
+    AggregateRealtimeConversion(prediction_types, segments, allocator, results);
+    AggregateUnigramPrediction(prediction_types, request, segments,
                                allocator, results);
-    AggregateBigramPrediction(prediction_type, request, segments,
+    AggregateBigramPrediction(prediction_types, request, segments,
                               allocator, results);
-    AggregateSuffixPrediction(prediction_type, request, segments,
+    AggregateSuffixPrediction(prediction_types, request, segments,
                               allocator, results);
+    AggregateEnglishPrediction(prediction_types, request, segments,
+                               allocator, results);
   }
 
   if (results->empty()) {
@@ -216,11 +242,11 @@ void DictionaryPredictor::RemovePrediction(const Segments &segments,
 }
 
 bool DictionaryPredictor::AddPredictionToCandidates(
+    const ConversionRequest &request,
     Segments *segments, vector<Result> *results) const {
   DCHECK(segments);
   DCHECK(results);
   const bool mixed_conversion = GET_REQUEST(mixed_conversion);
-
   const string &input_key = segments->conversion_segment(0).key();
   const size_t input_key_len = Util::CharsLen(input_key);
 
@@ -245,6 +271,8 @@ bool DictionaryPredictor::AddPredictionToCandidates(
   int added = 0;
   set<string> seen;
 
+  int added_suffix = 0;
+
   for (size_t i = 0; i < results->size(); ++i) {
     if (added >= size || results->at(i).cost == INT_MAX) {
       break;
@@ -255,7 +283,7 @@ bool DictionaryPredictor::AddPredictionToCandidates(
     const Node *node = result.node;
     DCHECK(node);
 
-    if (result.type == NO_PREDICTION) {
+    if (result.types == NO_PREDICTION) {
       continue;
     }
 
@@ -263,24 +291,24 @@ bool DictionaryPredictor::AddPredictionToCandidates(
     // is true.
     // TODO(manabe): Add a unit test. For that, we'll need a mock class for
     //               SuppressionDictionary.
-    if (SuggestionFilter::IsBadSuggestion(node->value) &&
-        !(mixed_conversion && result.type & REALTIME)) {
+    if (suggestion_filter_->IsBadSuggestion(node->value) &&
+        !(mixed_conversion && result.types & REALTIME)) {
       continue;
     }
 
     // don't suggest exactly the same candidate as key.
     // if |mixed_conversion| is true, that's not the case.
     if (!mixed_conversion &&
-        !(result.type & REALTIME) &&
-        (((result.type & BIGRAM) &&
+        !(result.types & REALTIME) &&
+        (((result.types & BIGRAM) &&
           exact_bigram_key == node->value) ||
-         (!(result.type & BIGRAM) &&
+         (!(result.types & BIGRAM) &&
           input_key == node->value))) {
       continue;
     }
 
     string key, value;
-    if (result.type & BIGRAM) {
+    if (result.types & BIGRAM) {
       // remove the prefix of history key and history value.
       key = node->key.substr(history_key.size(),
                              node->key.size() - history_key.size());
@@ -303,6 +331,12 @@ bool DictionaryPredictor::AddPredictionToCandidates(
       continue;
     }
 
+    if (result.types == SUFFIX && added_suffix++ >= 20) {
+      // TODO(toshiyuki): Need refactoring for controlling suffix
+      // prediction number after we will fix the appropriate number.
+      continue;
+    }
+
     Segment::Candidate *candidate = segment->push_back_candidate();
     DCHECK(candidate);
 
@@ -317,30 +351,59 @@ bool DictionaryPredictor::AddPredictionToCandidates(
     candidate->cost = results->at(i).cost;
     if (node->attributes & Node::SPELLING_CORRECTION) {
       candidate->attributes |= Segment::Candidate::SPELLING_CORRECTION;
+    } else if (IsLatinInputMode(request)) {
+      candidate->attributes |= Segment::Candidate::NO_VARIANTS_EXPANSION;
+      candidate->attributes |= Segment::Candidate::NO_EXTRA_DESCRIPTION;
+    }
+    if (node->attributes & Node::NO_VARIANTS_EXPANSION) {
+      candidate->attributes |= Segment::Candidate::NO_VARIANTS_EXPANSION;
+    }
+    if (node->attributes & Node::USER_DICTIONARY) {
+      candidate->attributes |= Segment::Candidate::USER_DICTIONARY;
     }
 
-    // Don't provide any descriptions for dictionary suggests
-#ifdef _DEBUG
-    const char kRealtimeConversionDescription[] = "Real-time Conversion";
-    const char kDictionarySuggestDescription[] = "Dictionary Suggest";
-    if (result.type & REALTIME) {
-      candidate->description = kRealtimeConversionDescription;
-    } else {
-      candidate->description = kDictionarySuggestDescription;
-    }
-#endif
+#ifdef DEBUG
+    SetDebugDescription(result.types, &candidate->description);
+#endif  // DEBUG
+
     ++added;
   }
 
   return added > 0;
 }
 
+#ifdef DEBUG
+void DictionaryPredictor::SetDebugDescription(PredictionTypes types,
+                                              string *description) {
+  vector<string> descriptions;
+  if (!description->empty()) {
+    descriptions.push_back(*description);
+  }
+  if (types & UNIGRAM) {
+    descriptions.push_back("Unigram");
+  }
+  if (types & BIGRAM) {
+    descriptions.push_back("Bigram");
+  }
+  if (types & REALTIME) {
+    descriptions.push_back("Realtime");
+  }
+  if (types & SUFFIX) {
+    descriptions.push_back("Suffix");
+  }
+  if (types & ENGLISH) {
+    descriptions.push_back("English");
+  }
+  Util::JoinStrings(descriptions, " ", description);
+}
+#endif  // DEBUG
+
 // return transition_cost[rid][node.lid] + node.wcost (+ penalties).
-int DictionaryPredictor::GetLMCost(PredictionType type,
+int DictionaryPredictor::GetLMCost(PredictionTypes types,
                                    const Node &node,
                                    int rid) const {
   int lm_cost = connector_->GetTransitionCost(rid, node.lid) + node.wcost;
-  if (!(type & REALTIME)) {
+  if (!(types & REALTIME)) {
     // Relatime conversion already adds perfix/suffix penalties to the nodes.
     // Note that we don't add prefix penalty the role of "bunsetsu" is
     // ambigous on zero-query suggestion.
@@ -368,20 +431,22 @@ const Node *DictionaryPredictor::LookupKeyValueFromDictionary(
 }
 
 bool DictionaryPredictor::GetHistoryKeyAndValue(
-    const Segments &segments,
-    string *key, string *value) const {
+    const Segments &segments, string *key, string *value) const {
   DCHECK(key);
   DCHECK(value);
-  if (segments.history_segments_size() > 0) {
-    const Segment &history_segment =
-        segments.history_segment(segments.history_segments_size() - 1);
-    if (history_segment.candidates_size() > 0) {
-      key->assign(history_segment.candidate(0).key);
-      value->assign(history_segment.candidate(0).value);
-      return true;
-    }
+  if (segments.history_segments_size() == 0) {
+    return false;
   }
-  return false;
+
+  const Segment &history_segment =
+      segments.history_segment(segments.history_segments_size() - 1);
+  if (history_segment.candidates_size() == 0) {
+    return false;
+  }
+
+  key->assign(history_segment.candidate(0).key);
+  value->assign(history_segment.candidate(0).value);
+  return true;
 }
 
 void DictionaryPredictor::SetPredictionCost(const Segments &segments,
@@ -411,12 +476,12 @@ void DictionaryPredictor::SetPredictionCost(const Segments &segments,
 
   for (size_t i = 0; i < results->size(); ++i) {
     const Node *node = (*results)[i].node;
-    const PredictionType type = (*results)[i].type;
-    const int32 cost = GetLMCost(type, *node, rid);
+    const PredictionTypes types = (*results)[i].types;
+    const int cost = GetLMCost(types, *node, rid);
     DCHECK(node);
 
     const size_t query_len =
-        ((*results)[i].type & BIGRAM) ? bigram_key_len : unigram_key_len;
+        (types & BIGRAM) ? bigram_key_len : unigram_key_len;
     const size_t key_len = Util::CharsLen(node->key);
 
     if (IsAggressiveSuggestion(query_len, key_len, cost,
@@ -490,13 +555,13 @@ void DictionaryPredictor::SetLMCost(const Segments &segments,
 
   for (size_t i = 0; i < results->size(); ++i) {
     const Node *node = (*results)[i].node;
-    const PredictionType type = (*results)[i].type;
+    const PredictionTypes types = (*results)[i].types;
     DCHECK(node);
-    int cost = GetLMCost(type, *node, rid);
+    int cost = GetLMCost(types, *node, rid);
     // Make exact candidates to have higher ranking.
     // Because for mobile, suggestion is the main candidates and
     // users expect the candidates for the input key on the candidates.
-    if (type & UNIGRAM) {
+    if (types & UNIGRAM) {
       const size_t input_key_len = Util::CharsLen(
           segments.conversion_segment(0).key());
       const size_t key_len = Util::CharsLen(node->key);
@@ -509,7 +574,7 @@ void DictionaryPredictor::SetLMCost(const Segments &segments,
         cost += kNotExactPenalty;
       }
     }
-    if (type & BIGRAM) {
+    if (types & BIGRAM) {
       // When user inputs "六本木" and there is an entry
       // "六本木ヒルズ" in the dictionary, we can suggest
       // "ヒルズ" as a ZeroQuery suggestion. In this case,
@@ -521,7 +586,9 @@ void DictionaryPredictor::SetLMCost(const Segments &segments,
       // the cost is basically the same as the cost between
       // "名詞,一般" and "名詞,一般".
       const int kDefaultTransitionCost = 1347;
-      cost += (kDefaultTransitionCost - prev_cost);
+      // Promoting bigram candidates.
+      const int kBigramBonus = 800;  // ~= 500*ln(5)
+      cost += (kDefaultTransitionCost - kBigramBonus - prev_cost);
     }
     (*results)[i].cost = cost;
   }
@@ -617,27 +684,26 @@ void DictionaryPredictor::RemoveMissSpelledCandidates(
 
     // delete same_key_index and same_value_index
     if (!same_key_index.empty() && !same_value_index.empty()) {
-      (*results)[i].type = NO_PREDICTION;
+      (*results)[i].types = NO_PREDICTION;
       for (size_t k = 0; k < same_key_index.size(); ++k) {
-        (*results)[same_key_index[k]].type = NO_PREDICTION;
+        (*results)[same_key_index[k]].types = NO_PREDICTION;
       }
     } else if (same_key_index.empty() && !same_value_index.empty()) {
-      (*results)[i].type = NO_PREDICTION;
+      (*results)[i].types = NO_PREDICTION;
     } else if (!same_key_index.empty() && same_value_index.empty()) {
       for (size_t k = 0; k < same_key_index.size(); ++k) {
-        (*results)[same_key_index[k]].type = NO_PREDICTION;
+        (*results)[same_key_index[k]].types = NO_PREDICTION;
       }
       if (request_key_len <=
-          GetMissSpelledPosition(result.node->key,
-                                 result.node->value)) {
-        (*results)[i].type = NO_PREDICTION;
+          GetMissSpelledPosition(result.node->key, result.node->value)) {
+        (*results)[i].types = NO_PREDICTION;
       }
     }
   }
 }
 
 bool DictionaryPredictor::IsAggressiveSuggestion(
-    size_t query_len, size_t key_len, int32 cost,
+    size_t query_len, size_t key_len, int cost,
     bool is_suggestion, size_t total_candidates_size) const {
   // Temporal workaround for fixing the problem where longer sentence-like
   // suggestions are shown when user input is very short.
@@ -699,11 +765,11 @@ size_t DictionaryPredictor::GetRealtimeCandidateMaxSize(
 }
 
 void DictionaryPredictor::AggregateRealtimeConversion(
-    PredictionType type,
+    PredictionTypes types,
     Segments *segments,
     NodeAllocatorInterface *allocator,
     vector<Result> *results) const {
-  if (!(type & REALTIME)) {
+  if (!(types & REALTIME)) {
     return;
   }
 
@@ -752,6 +818,12 @@ void DictionaryPredictor::AggregateRealtimeConversion(
       if (candidate.attributes & Segment::Candidate::SPELLING_CORRECTION) {
         node->attributes |= Node::SPELLING_CORRECTION;
       }
+      if (candidate.attributes & Segment::Candidate::NO_VARIANTS_EXPANSION) {
+        node->attributes |= Node::NO_VARIANTS_EXPANSION;
+      }
+      if (candidate.attributes & Segment::Candidate::USER_DICTIONARY) {
+        node->attributes |= Node::USER_DICTIONARY;
+      }
       results->push_back(Result(node, REALTIME));
     }
     // remove candidates created by ImmutableConverter.
@@ -767,13 +839,9 @@ void DictionaryPredictor::AggregateRealtimeConversion(
 }
 
 size_t DictionaryPredictor::GetUnigramCandidateCutoffThreshold(
-    const Segments &segments,
-    bool mixed_conversion) const {
+    const Segments &segments) const {
   DCHECK(segments.request_type() == Segments::PREDICTION ||
          segments.request_type() == Segments::SUGGESTION);
-  if (mixed_conversion) {
-    return kSuggestionMaxNodesSize;
-  }
   if (segments.request_type() == Segments::PREDICTION) {
     // If PREDICTION, many candidates are needed than SUGGESTION.
     return kPredictionMaxNodesSize;
@@ -782,32 +850,47 @@ size_t DictionaryPredictor::GetUnigramCandidateCutoffThreshold(
 }
 
 void DictionaryPredictor::AggregateUnigramPrediction(
-    PredictionType type,
+    PredictionTypes types,
     const ConversionRequest &request,
     Segments *segments,
     NodeAllocatorInterface *allocator,
     vector<Result> *results) const {
-  if (!(type & UNIGRAM)) {
+  if (!(types & UNIGRAM)) {
     return;
   }
 
   DCHECK(segments);
-  DCHECK(results);
-  DCHECK(dictionary_);
   DCHECK(allocator);
-  DCHECK(!segments->conversion_segment(0).key().empty());
+  DCHECK(results);
+  DCHECK(segments->request_type() == Segments::PREDICTION ||
+         segments->request_type() == Segments::SUGGESTION);
 
   const bool mixed_conversion = GET_REQUEST(mixed_conversion);
-  const size_t cutoff_threshold = GetUnigramCandidateCutoffThreshold(
-      *segments,
-      mixed_conversion);
+  if (!mixed_conversion) {
+    AggregateUnigramCandidate(request, segments, allocator, results);
+  } else {
+    AggregateUnigramCandidateForMixedConversion(
+        request, segments, allocator, results);
+  }
+}
+
+void DictionaryPredictor::AggregateUnigramCandidate(
+    const ConversionRequest &request,
+    Segments *segments,
+    NodeAllocatorInterface *allocator,
+    vector<Result> *results) const {
+  DCHECK(segments);
+  DCHECK(allocator);
+  DCHECK(results);
+  DCHECK(dictionary_);
+
+  const size_t cutoff_threshold = GetUnigramCandidateCutoffThreshold(*segments);
   allocator->set_max_nodes_size(cutoff_threshold);
-
-  const size_t prev_results_size = results->size();
-
   // no history key
   const Node *unigram_node = GetPredictiveNodes(
       dictionary_, "", request, *segments, allocator);
+
+  const size_t prev_results_size = results->size();
   size_t unigram_results_size = 0;
   for (; unigram_node != NULL; unigram_node = unigram_node->bnext) {
     results->push_back(Result(unigram_node, UNIGRAM));
@@ -823,13 +906,92 @@ void DictionaryPredictor::AggregateUnigramPrediction(
   }
 }
 
-void DictionaryPredictor::AggregateBigramPrediction(
-    PredictionType type,
+void DictionaryPredictor::AggregateUnigramCandidateForMixedConversion(
     const ConversionRequest &request,
     Segments *segments,
     NodeAllocatorInterface *allocator,
     vector<Result> *results) const {
-  if (!(type & BIGRAM)) {
+  const size_t cutoff_threshold = kPredictionMaxNodesSize;
+  allocator->set_max_nodes_size(cutoff_threshold);
+
+  // No history key
+  const Node *unigram_node = GetPredictiveNodes(
+      dictionary_, "", request, *segments, allocator);
+
+  // Move pointers to a vector.
+  // TODO(hidehiko): reserve the nodes' size, when we make an interface
+  //   to return the number of returned nodes by dictionary look-up.
+  vector<const Node*> nodes;
+  for (; unigram_node != NULL; unigram_node = unigram_node->bnext) {
+    nodes.push_back(unigram_node);
+  }
+
+  // Hereafter, we split "Needed Nodes" and "(maybe) Unneeded Nodes."
+  // The algorithm is:
+  // 1) Take the Node with minimum cost.
+  // 2) Remove nodes which is "redundant" (defined by MaybeRedundant),
+  //    from remaining nodes.
+  // 3) Repeat 1) and 2) five times.
+  // Note: to reduce the number of memory allocation, we swap out the
+  //   "redundant" nodes to the end of the |nodes| vector.
+  const size_t kDeleteTrialNum = 5;
+  typedef vector<const Node *>::iterator Iter;
+
+  // min_iter is the beginning of the remaining nodes (inclusive), and
+  // max_iter is the end of the remaining nodes (exclusive).
+  Iter min_iter = nodes.begin();
+  Iter max_iter = nodes.end();
+  for (size_t i = 0; i < kDeleteTrialNum; ++i) {
+    if (min_iter == max_iter) {
+      break;
+    }
+
+    // Find the Node with minimum cost. Swap it with the beginning element.
+    iter_swap(min_iter, min_element(min_iter, max_iter, NodeLess()));
+    const Node *reference_node = *min_iter;
+
+    // Preserve the reference node.
+    ++min_iter;
+
+    // Traverse all remaining elements and check if each node is redundant.
+    for (Iter iter = min_iter; iter != max_iter; ) {
+      if (MaybeRedundant(reference_node, *iter)) {
+        // Swap out the redundant node.
+        --max_iter;
+        iter_swap(iter, max_iter);
+      } else {
+        ++iter;
+      }
+    }
+  }
+
+  // Then the |nodes| contains;
+  // [begin, min_iter): reference nodes in the above loop.
+  // [max_iter, end): (maybe) redundant nodes.
+  // [min_iter, max_iter): remaining nodes.
+  // Here, we revive the redundant nodes up to five in the node cost order.
+  const size_t kDoNotDeleteNum = 5;
+  if (distance(max_iter, nodes.end()) >= kDoNotDeleteNum) {
+    partial_sort(max_iter, max_iter + kDoNotDeleteNum, nodes.end());
+    max_iter += kDoNotDeleteNum;
+  } else {
+    max_iter = nodes.end();
+  }
+
+  // Finally output the result.
+  results->reserve(results->size() + distance(nodes.begin(), max_iter));
+  for (Iter iter = nodes.begin(); iter != max_iter; ++iter) {
+    results->push_back(Result(*iter, UNIGRAM));
+  }
+}
+
+void DictionaryPredictor::AggregateBigramPrediction(
+    PredictionTypes types,
+    const ConversionRequest &request,
+    Segments *segments,
+    NodeAllocatorInterface *allocator,
+    vector<Result> *results) const {
+  if (!(types & BIGRAM)) {
     return;
   }
 
@@ -838,12 +1000,23 @@ void DictionaryPredictor::AggregateBigramPrediction(
   DCHECK(dictionary_);
   DCHECK(allocator);
 
-  const string &input_key = segments->conversion_segment(0).key();
-  const bool is_zero_query = input_key.empty();
-
+  // TODO(toshiyuki): Support suggestion from the last 2 histories.
+  //  ex) 六本木+ヒルズ->レジデンス
   string history_key, history_value;
-  GetHistoryKeyAndValue(*segments, &history_key, &history_value);
+  if (!GetHistoryKeyAndValue(*segments, &history_key, &history_value)) {
+    return;
+  }
+  AddBigramResultsFromHistory(
+      history_key, history_value, request, segments, allocator, results);
+}
 
+void DictionaryPredictor::AddBigramResultsFromHistory(
+    const string &history_key,
+    const string &history_value,
+    const ConversionRequest &request,
+    Segments *segments,
+    NodeAllocatorInterface *allocator,
+    vector<Result> *results) const {
   // Check that history_key/history_value are in the dictionary.
   const Node *history_node = LookupKeyValueFromDictionary(
       history_key, history_value, allocator);
@@ -868,7 +1041,8 @@ void DictionaryPredictor::AggregateBigramPrediction(
   for (; bigram_node != NULL; bigram_node = bigram_node->bnext) {
     // filter out the output (value)'s prefix doesn't match to
     // the history value.
-    if (Util::StartsWith(bigram_node->value, history_value)) {
+    if (Util::StartsWith(bigram_node->value, history_value) &&
+        (bigram_node->value.size() > history_value.size())) {
       results->push_back(Result(bigram_node, BIGRAM));
       ++bigram_results_size;
     }
@@ -889,65 +1063,89 @@ void DictionaryPredictor::AggregateBigramPrediction(
     return;
   }
 
+  const Util::ScriptType history_ctype = Util::GetScriptType(history_value);
   const Util::ScriptType last_history_ctype =
       Util::GetScriptType(Util::SubString(history_value,
                                           history_value_size - 1, 1));
-
-  // Filter out irrelevant bigrams. For example, we don't want to
-  // suggest "リカ" from the history "アメ".
   for (size_t i = prev_results_size; i < results->size(); ++i) {
-    const Node *node = (*results)[i].node;
-    DCHECK(node);
-    const string key = node->key.substr(history_key.size(),
-                                        node->key.size() -
-                                        history_key.size());
-    const string value = node->value.substr(history_value.size(),
-                                            node->value.size() -
-                                            history_value.size());
-    // Don't suggest 0-length key/value.
-    if (key.empty() || value.empty()) {
-      (*results)[i].type = NO_PREDICTION;
-      continue;
-    }
+    CheckBigramResult(history_node, history_ctype, last_history_ctype,
+                      allocator, &(*results)[i]);
+  }
+}
 
-    // If freq("アメ") < freq("アメリカ"), we don't
-    // need to suggest it. As "アメリカ" should already be
-    // suggested when user type "アメ".
-    // Note that wcost = -500 * log(prob).
-    if (history_node->wcost > node->wcost) {
-      (*results)[i].type = NO_PREDICTION;
-      continue;
-    }
+// Filter out irrelevant bigrams. For example, we don't want to
+// suggest "リカ" from the history "アメ".
+void DictionaryPredictor::CheckBigramResult(
+    const Node *history_node,
+    const Util::ScriptType history_ctype,
+    const Util::ScriptType last_history_ctype,
+    NodeAllocatorInterface *allocator,
+    Result *result) const {
+  DCHECK(history_node);
+  DCHECK(result);
 
-    // If character type doesn't change, this boundary might NOT
-    // be a word boundary. If character type is HIRAGANA,
-    // we don't trust it. If Katakana, only trust iif the
-    // entire key is reasonably long.
-    const Util::ScriptType ctype =
-        Util::GetScriptType(Util::SubString(value, 0, 1));
-    if (ctype == last_history_ctype &&
-        (ctype == Util::HIRAGANA ||
-         (ctype == Util::KATAKANA && Util::CharsLen(node->key) <= 5))) {
-      (*results)[i].type = NO_PREDICTION;
-      continue;
-    }
+  const Node *node = result->node;
+  DCHECK(node);
+  const string &history_key = history_node->key;
+  const string &history_value = history_node->value;
+  const string key(node->key, history_key.size(),
+                   node->key.size() - history_key.size());
+  const string value(node->value, history_value.size(),
+                     node->value.size() - history_value.size());
 
-    // The suggested key/value pair must exist in the dictionary.
-    // For example, we don't want to suggest "ターネット" from
-    // the history "イン".
-    // If character type is Kanji and the suggestion is not a
-    // zero_query_suggestion, we relax this condition, as there are
-    // many Kanji-compounds which may not in the dictionary. For example,
-    // we want to suggest "霊長類研究所" from the history "京都大学".
-    if (ctype == Util::KANJI && is_zero_query) {
-      // Do not filter this.
-      continue;
-    }
+  // Don't suggest 0-length key/value.
+  if (key.empty() || value.empty()) {
+    result->types = NO_PREDICTION;
+    return;
+  }
 
-    if (NULL == LookupKeyValueFromDictionary(key, value, allocator)) {
-      (*results)[i].type = NO_PREDICTION;
-      continue;
-    }
+  const Util::ScriptType ctype =
+      Util::GetScriptType(Util::SubString(value, 0, 1));
+
+  if (history_ctype == Util::KANJI &&
+      ctype == Util::KATAKANA) {
+    // Do not filter "六本木ヒルズ"
+    return;
+  }
+
+  // If freq("アメ") < freq("アメリカ"), we don't
+  // need to suggest it. As "アメリカ" should already be
+  // suggested when user type "アメ".
+  // Note that wcost = -500 * log(prob).
+  if (ctype != Util::KANJI &&
+      history_node->wcost > node->wcost) {
+    result->types = NO_PREDICTION;
+    return;
+  }
+
+  // If character type doesn't change, this boundary might NOT
+  // be a word boundary. If character type is HIRAGANA,
+  // we don't trust it. If Katakana, only trust iif the
+  // entire key is reasonably long.
+  if (ctype == last_history_ctype &&
+      (ctype == Util::HIRAGANA ||
+       (ctype == Util::KATAKANA && Util::CharsLen(node->key) <= 5))) {
+    result->types = NO_PREDICTION;
+    return;
+  }
+
+  // The suggested key/value pair must exist in the dictionary.
+  // For example, we don't want to suggest "ターネット" from
+  // the history "イン".
+  // If character type is Kanji and the suggestion is not a
+  // zero_query_suggestion, we relax this condition, as there are
+  // many Kanji-compounds which may not in the dictionary. For example,
+  // we want to suggest "霊長類研究所" from the history "京都大学".
+  if (ctype == Util::KANJI && Util::CharsLen(value) >= 2) {
+    // Do not filter this.
+    // TODO(toshiyuki): one-length kanji prediciton may be annoying other than
+    // some exceptions, "駅", "口", etc
+    return;
+  }
+
+  if (NULL == LookupKeyValueFromDictionary(key, value, allocator)) {
+    result->types = NO_PREDICTION;
+    return;
   }
 }
 
@@ -990,13 +1188,68 @@ const Node *DictionaryPredictor::GetPredictiveNodes(
   }
 }
 
+const Node *DictionaryPredictor::GetPredictiveNodesForEnglish(
+    const DictionaryInterface *dictionary,
+    const string &history_key,
+    const ConversionRequest &request,
+    const Segments &segments,
+    NodeAllocatorInterface *allocator) const {
+  if (!request.has_composer()) {
+    const string input_key = history_key + segments.conversion_segment(0).key();
+    return dictionary->LookupPredictive(input_key.c_str(),
+                                        input_key.size(),
+                                        allocator);
+  }
+
+  string input_key;
+  request.composer().GetQueryForPrediction(&input_key);
+  // We don't look up English words when key length is one.
+  if (input_key.size() < 2) {
+    return NULL;
+  }
+  Node *head = NULL;
+  if (Util::IsUpperAscii(input_key)) {
+    // For upper case key, look up its lower case version and then transform the
+    // results to upper case.
+    string key(input_key);
+    Util::LowerString(&key);
+    head = dictionary->LookupPredictive(key.c_str(), key.size(), allocator);
+    for (Node *node = head; node != NULL; node = node->bnext) {
+      Util::UpperString(&node->value);
+    }
+  } else if (Util::IsCapitalizedAscii(input_key)) {
+    // For capitalized key, look up its lower case version and then transform
+    // the results to capital.
+    string key(input_key);
+    Util::LowerString(&key);
+    head = dictionary->LookupPredictive(key.c_str(), key.size(), allocator);
+    for (Node *node = head; node != NULL; node = node->bnext) {
+      Util::CapitalizeString(&node->value);
+    }
+  } else {
+    // For other cases (lower and as-is), just look up directly.
+    head = dictionary->LookupPredictive(input_key.c_str(),
+                                        input_key.size(),
+                                        allocator);
+  }
+  // If input mode is FULL_ASCII, then convert the results to full-width.
+  if (request.composer().GetInputMode() == transliteration::FULL_ASCII) {
+    string tmp;
+    for (Node *node = head; node != NULL; node = node->bnext) {
+      tmp.assign(node->value);
+      Util::HalfWidthAsciiToFullWidthAscii(tmp, &node->value);
+    }
+  }
+  return head;
+}
+
 void DictionaryPredictor::AggregateSuffixPrediction(
-    PredictionType type,
+    PredictionTypes types,
     const ConversionRequest &request,
     Segments *segments,
     NodeAllocatorInterface *allocator,
     vector<Result> *results) const {
-  if (!(type & SUFFIX)) {
+  if (!(types & SUFFIX)) {
     return;
   }
 
@@ -1008,7 +1261,7 @@ void DictionaryPredictor::AggregateSuffixPrediction(
   if (history_size) {
     const string &history_key =
         segments->history_segment(history_size - 1).key();
-    has_number_history = IsNumber(history_key);
+    has_number_history = NumberUtil::IsDecimalInteger(history_key);
   }
 
   if (has_number_history && segments->conversion_segment(0).key().size() == 0) {
@@ -1039,14 +1292,61 @@ void DictionaryPredictor::AggregateSuffixPrediction(
       cost += kSuffixPenalty;
     }
   } else {
+    const string kEmptyHistoryKey = "";
     const Node *node = GetPredictiveNodes(
-        suffix_dictionary_, "", request, *segments, allocator);
+        suffix_dictionary_, kEmptyHistoryKey, request, *segments, allocator);
     for (; node != NULL; node = node->bnext) {
+      if (Util::CharsLen(node->value) <= 1) {
+        // Do not suggest one char suffix.
+        // TODO(toshiyuki): If this works well, fix from the dictionary
+        // generation process.
+        continue;
+      }
       results->push_back(Result(node, SUFFIX));
     }
   }
 }
 
+void DictionaryPredictor::AggregateEnglishPrediction(
+    PredictionTypes types,
+    const ConversionRequest &request,
+    Segments *segments,
+    NodeAllocatorInterface *allocator,
+    vector<Result> *results) const {
+  if (!(types & ENGLISH)) {
+    return;
+  }
+  DCHECK(segments);
+  DCHECK(allocator);
+  DCHECK(results);
+  DCHECK(dictionary_);
+
+  const size_t cutoff_threshold = GetUnigramCandidateCutoffThreshold(*segments);
+  allocator->set_max_nodes_size(cutoff_threshold);
+
+  // Currently, history key is never utilized.
+  // TODO(noriyukit): Come up with a way of utilizing it.
+  const string kEmptyHistoryKey = "";
+  const Node *unigram_node = GetPredictiveNodesForEnglish(
+      dictionary_, kEmptyHistoryKey, request, *segments, allocator);
+
+  const size_t prev_results_size = results->size();
+  size_t unigram_results_size = 0;
+  for (; unigram_node != NULL; unigram_node = unigram_node->bnext) {
+    results->push_back(Result(unigram_node, ENGLISH));
+    // If size reaches max_nodes_size (== cutoff_threshold).  we don't show the
+    // candidates, since disambiguation from 256 candidates is hard. (It may
+    // exceed max_nodes_size, because this is just a limit for each backend, so
+    // total number may be larger.)
+    // TODO(team): If DictionaryInterface provides the information of size of
+    // returned node list, we can avoid unnecessary push_backs and resize here.
+    if (++unigram_results_size >= allocator->max_nodes_size()) {
+      results->resize(prev_results_size);
+      return;
+    }
+  }
+}
+
 bool DictionaryPredictor::IsZipCodeRequest(const string &key) const {
   if (key.empty()) {
     return false;
@@ -1069,8 +1369,8 @@ bool DictionaryPredictor::IsZipCodeRequest(const string &key) const {
   return true;
 }
 
-DictionaryPredictor::PredictionType
-DictionaryPredictor::GetPredictionType(const Segments &segments) const {
+DictionaryPredictor::PredictionTypes DictionaryPredictor::GetPredictionTypes(
+    const ConversionRequest &request, const Segments &segments) const {
   if (segments.request_type() == Segments::CONVERSION) {
     VLOG(2) << "request type is CONVERSION";
     return NO_PREDICTION;
@@ -1081,10 +1381,14 @@ DictionaryPredictor::GetPredictionType(const Segments &segments) const {
     return NO_PREDICTION;
   }
 
+  if (IsLatinInputMode(request)) {
+    return ENGLISH | REALTIME;
+  }
+
   const string &key = segments.conversion_segment(0).key();
 
   // default setting
-  int result = NO_PREDICTION;
+  PredictionTypes result = NO_PREDICTION;
 
   // support realtime conversion.
   const size_t kMaxKeySize = 300;   // 300 bytes in UTF8
@@ -1101,14 +1405,14 @@ DictionaryPredictor::GetPredictionType(const Segments &segments) const {
   if (!GET_CONFIG(use_dictionary_suggest) &&
       segments.request_type() == Segments::SUGGESTION) {
     VLOG(2) << "no_dictionary_suggest";
-    return static_cast<PredictionType>(result);
+    return result;
   }
 
   const bool zero_query_suggestion = GET_REQUEST(zero_query_suggestion);
 
   const size_t key_len = Util::CharsLen(key);
   if (key_len == 0 && !zero_query_suggestion) {
-    return static_cast<PredictionType>(result);
+    return result;
   }
 
   // Never trigger prediction if key looks like zip code.
@@ -1116,7 +1420,7 @@ DictionaryPredictor::GetPredictionType(const Segments &segments) const {
 
   if (segments.request_type() == Segments::SUGGESTION &&
       is_zip_code && key_len < 6) {
-    return static_cast<PredictionType>(result);
+    return result;
   }
 
   const int kMinUnigramKeyLen = zero_query_suggestion ? 1 : 3;
@@ -1151,6 +1455,6 @@ DictionaryPredictor::GetPredictionType(const Segments &segments) const {
     result |= SUFFIX;
   }
 
-  return static_cast<PredictionType>(result);
+  return result;
 }
 }  // namespace mozc
