@@ -31,6 +31,7 @@
 
 #include <sstream>
 
+#include "base/number_util.h"
 #include "base/util.h"
 #include "client/client_interface.h"
 #include "renderer/table_layout_mock.h"
@@ -108,7 +109,7 @@ void SetTestCandidates(uint32 count,
     candidate->set_index(i);
     candidate->set_id(i * 0x10);
 
-    const string id_str = Util::SimpleItoa(i);
+    const string id_str = NumberUtil::SimpleItoa(i);
 
     if (has_value) {
       candidate->set_value(kSampleValue + id_str);
@@ -137,7 +138,7 @@ void SetTestCandidates(uint32 count,
 string GetExpectedValue(int index, bool has_prefix, bool has_suffix) {
   string result;
 
-  const string id_str = Util::SimpleItoa(index);
+  const string id_str = NumberUtil::SimpleItoa(index);
   const string expected_prefix = kSamplePrefix + id_str;
   const string expected_value = kSampleValue + id_str;
   const string expected_suffix = kSampleSuffix + id_str;
@@ -152,12 +153,12 @@ string GetExpectedValue(int index, bool has_prefix, bool has_suffix) {
 }
 
 string GetExpectedShortcut(int index) {
-  const string id_str = Util::SimpleItoa(index);
+  const string id_str = NumberUtil::SimpleItoa(index);
   return kSampleShortcut + id_str;
 }
 
 string GetExpectedDescription(int index) {
-  const string id_str = Util::SimpleItoa(index);
+  const string id_str = NumberUtil::SimpleItoa(index);
   return kSampleDescription + id_str;
 }
 
