@@ -39,8 +39,10 @@
 #include <set>
 #include <string>
 #include <vector>
+
 #include "base/base.h"
 #include "base/file_stream.h"
+#include "base/logging.h"
 #include "base/util.h"
 
 DEFINE_string(usage_data_file, "", "usage data file");
@@ -71,7 +73,7 @@ bool UsageItemKeynameCmp(const UsageItem& l, const UsageItem& r) {
 void LoadConjugation(const string &filename,
                      map<string, vector<ConjugationType> > *output,
                      map<string, ConjugationType> *baseform_map) {
-  mozc::InputFileStream ifs(filename.c_str());
+  InputFileStream ifs(filename.c_str());
   CHECK(ifs);
 
   string line;
@@ -100,7 +102,7 @@ void LoadConjugation(const string &filename,
 void LoadUsage(const string &filename,
                vector<UsageItem> *usage_entries,
                vector<string> *conjugation_list) {
-  mozc::InputFileStream ifs(filename.c_str());
+  InputFileStream ifs(filename.c_str());
 
   if (!ifs) {
     LOG(WARNING) << "Can't open file:" << filename;
@@ -186,7 +188,7 @@ void Convert() {
 
   ostream *ofs = &cout;
   if (!FLAGS_output.empty()) {
-    ofs = new mozc::OutputFileStream(FLAGS_output.c_str());
+    ofs = new OutputFileStream(FLAGS_output.c_str());
   }
 
   *ofs << "// This header file is generated by "
