@@ -35,6 +35,9 @@
 #include <utility>
 #include <vector>
 
+#include "base/base.h"
+#include "base/logging.h"
+#include "base/number_util.h"
 #include "base/util.h"
 #include "config/config.pb.h"
 #include "config/config_handler.h"
@@ -47,13 +50,104 @@
 namespace mozc {
 namespace {
 
+// Rewrite type
+enum RewriteType {
+  NO_REWRITE = 0,
+  ARABIC_FIRST,  // arabic candidates first ordering
+  KANJI_FIRST,  // kanji candidates first ordering
+};
+
+struct RewriteCandidateInfo {
+  RewriteType type;
+  int position;
+  Segment::Candidate candidate;
+};
+
+bool IsNumber(const POSMatcher &pos_matcher, uint16 lid) {
+  // Number candidates sometimes categorized as general noun.
+  // TODO(toshiyuki): It's better if we can rewrite
+  // from general noun POS to number POS
+  // TODO(toshiyuki): We can remove general noun check if we can set
+  // correct POS.
+  return (pos_matcher.IsNumber(lid) || pos_matcher.IsKanjiNumber(lid) ||
+          pos_matcher.IsGeneralNoun(lid));
+}
+
+// Returns rewrite type for the given segment and base candidate information.
+// base_candidate_pos: the index of the base candidate.
+// *arabic_candidate: arabic candidate using numeric style conversion.
+// POS information, cost, etc will be copied from base candidate.
+RewriteType GetRewriteTypeAndBase(
+    const Segment &seg,
+    int base_candidate_pos,
+    const POSMatcher &pos_matcher,
+    Segment::Candidate *arabic_candidate) {
+  DCHECK(arabic_candidate);
+
+  const Segment::Candidate &c = seg.candidate(base_candidate_pos);
+  if (!IsNumber(pos_matcher, c.lid)) {
+    return NO_REWRITE;
+  }
+
+  if (Util::GetScriptType(c.content_value) == Util::NUMBER) {
+    arabic_candidate->CopyFrom(c);
+    return ARABIC_FIRST;
+  }
+
+  string half_width_new_content_value;
+  Util::FullWidthToHalfWidth(c.content_key, &half_width_new_content_value);
+  // Try to get normalized kanji_number and arabic_number.
+  // If it failed, do nothing.
+  // Retain suffix for later use.
+  string number_suffix, kanji_number, arabic_number;
+  if (!NumberUtil::NormalizeNumbersWithSuffix(c.content_value,
+                                              true,  // trim_reading_zeros
+                                              &kanji_number,
+                                              &arabic_number,
+                                              &number_suffix) ||
+      arabic_number == half_width_new_content_value) {
+    return NO_REWRITE;
+  }
+  const string suffix = c.value.substr(
+      c.content_value.size(), c.value.size() - c.content_value.size());
+  arabic_candidate->Init();
+  arabic_candidate->value = arabic_number + number_suffix + suffix;
+  arabic_candidate->content_value = arabic_number + number_suffix;
+  arabic_candidate->key = c.key;
+  arabic_candidate->content_key = c.content_key;
+  arabic_candidate->cost = c.cost;
+  arabic_candidate->structure_cost = c.structure_cost;
+  arabic_candidate->lid = c.lid;
+  arabic_candidate->rid = c.rid;
+  return KANJI_FIRST;
+}
+
+void GetRewriteCandidateInfos(
+    const Segment &seg,
+    const POSMatcher &pos_matcher,
+    vector<RewriteCandidateInfo> *rewrite_candidate_info) {
+  DCHECK(rewrite_candidate_info);
+  RewriteCandidateInfo info;
+
+  for (size_t i = 0; i < seg.candidates_size(); ++i) {
+    const RewriteType type = GetRewriteTypeAndBase(
+        seg, i, pos_matcher, &info.candidate);
+    if (type == NO_REWRITE) {
+      continue;
+    }
+    info.type = type;
+    info.position = i;
+    rewrite_candidate_info->push_back(info);
+  }
+}
+
 // If top candidate is Kanji numeric, we want to expand at least
 // 5 candidates apart from base candidate.
 // http://b/issue?id=2872048
 const int kArabicNumericOffset = 5;
 
 void PushBackCandidate(const string &value, const string &desc,
-                       Util::NumberString::Style style,
+                       NumberUtil::NumberString::Style style,
                        vector<Segment::Candidate> *results) {
   bool found = false;
   for (vector<Segment::Candidate>::const_iterator it = results->begin();
@@ -101,30 +195,30 @@ class CheckValueOperator {
 // delete them.
 // TODO(toshiyuki): Delete candidates between base pos and insert pos
 // if necessary.
-void EraseExistingCandidates(const vector<Segment::Candidate> &results,
-                             int *base_candidate_pos,
-                             int *insert_pos, Segment *seg) {
-  DCHECK(base_candidate_pos);
-  DCHECK(insert_pos);
+void EraseExistingCandidates(
+    const vector<Segment::Candidate> &results,
+    int base_candidate_pos,
+    Segment *seg,
+    vector<RewriteCandidateInfo> *rewrite_candidate_info_list) {
   DCHECK(seg);
   // Remember base candidate value
-  const string &base_value = seg->candidate(*base_candidate_pos).value;
-  size_t pos = 0;
-  while (pos < seg->candidates_size()) {
-    const string &value = seg->candidate(pos).value;
-    if (value == base_value) {
-      break;
-    }
+  for (int pos = base_candidate_pos - 1; pos >= 0; --pos) {
     // Simple liner search. |results| size is small. (at most 10 or so)
-    const vector<Segment::Candidate>::const_iterator itr =
-        find_if(results.begin(), results.end(), CheckValueOperator(value));
-    if (itr == results.end()) {
-      ++pos;
+    const vector<Segment::Candidate>::const_iterator iter =
+        find_if(results.begin(), results.end(),
+                CheckValueOperator(seg->candidate(pos).value));
+    if (iter == results.end()) {
       continue;
     }
+
     seg->erase_candidate(pos);
-    --(*base_candidate_pos);
-    --(*insert_pos);
+
+    // Adjust position in rewrite_candidate_info.
+    for (size_t i = 0; i < rewrite_candidate_info_list->size(); ++i) {
+      if ((*rewrite_candidate_info_list)[i].position > pos) {
+        --(*rewrite_candidate_info_list)[i].position;
+      }
+    }
   }
 }
 
@@ -144,9 +238,9 @@ void MergeCandidateInfoInternal(const Segment::Candidate &base_cand,
   cand->style = result_cand.style;
   cand->description = result_cand.description;
   // Don't want to have FULL_WIDTH form for Hex/Oct/BIN..etc.
-  if (cand->style == Util::NumberString::NUMBER_HEX ||
-      cand->style == Util::NumberString::NUMBER_OCT ||
-      cand->style == Util::NumberString::NUMBER_BIN) {
+  if (cand->style == NumberUtil::NumberString::NUMBER_HEX ||
+      cand->style == NumberUtil::NumberString::NUMBER_OCT ||
+      cand->style == NumberUtil::NumberString::NUMBER_BIN) {
     cand->attributes |= Segment::Candidate::NO_VARIANTS_EXPANSION;
   }
 }
@@ -202,8 +296,8 @@ void InsertConvertedCandidates(const vector<Segment::Candidate> &results,
     vector<Segment::Candidate>::const_iterator itr =
         find_if(results.begin(), results.end(), CheckValueOperator(base_value));
     if (itr != results.end() &&
-        itr->style != Util::NumberString::NUMBER_KANJI &&
-        itr->style != Util::NumberString::NUMBER_KANJI_ARABIC) {
+        itr->style != NumberUtil::NumberString::NUMBER_KANJI &&
+        itr->style != NumberUtil::NumberString::NUMBER_KANJI_ARABIC) {
       // Update exsisting base candidate
       UpdateCandidate(seg, base_candidate_pos, base_cand, results[0]);
     } else {
@@ -219,9 +313,8 @@ void InsertConvertedCandidates(const vector<Segment::Candidate> &results,
   }
 }
 
-int GetInsertPos(int base_pos, const Segment &segment,
-                 NumberRewriter::RewriteType type) {
-  if (type == NumberRewriter::ARABIC_FIRST) {
+int GetInsertPos(int base_pos, const Segment &segment, RewriteType type) {
+  if (type == ARABIC_FIRST) {
     // +2 for arabic half_width full_width expansion
     return min(base_pos + 2, static_cast<int>(segment.candidates_size()));
   } else {
@@ -231,61 +324,41 @@ int GetInsertPos(int base_pos, const Segment &segment,
 }
 
 void InsertHalfArabic(const string &half_arabic,
-                      vector<Util::NumberString> *output) {
-  output->push_back(Util::NumberString(half_arabic, "",
-                                       Util::NumberString::DEFAULT_STYLE));
+                      vector<NumberUtil::NumberString> *output) {
+  output->push_back(
+      NumberUtil::NumberString(half_arabic, "",
+                               NumberUtil::NumberString::DEFAULT_STYLE));
 }
 
-void GetNumbers(NumberRewriter::RewriteType type,
-                bool exec_radix_conversion,
+void GetNumbers(RewriteType type, bool exec_radix_conversion,
                 const string &arabic_content_value,
-                vector<Util::NumberString> *output) {
+                vector<NumberUtil::NumberString> *output) {
   DCHECK(output);
-  if (type == NumberRewriter::ARABIC_FIRST) {
+  if (type == ARABIC_FIRST) {
     InsertHalfArabic(arabic_content_value, output);
-    Util::ArabicToWideArabic(arabic_content_value, output);
-    Util::ArabicToSeparatedArabic(arabic_content_value, output);
-    Util::ArabicToKanji(arabic_content_value, output);
-    Util::ArabicToOtherForms(arabic_content_value, output);
-  } else if (type == NumberRewriter::KANJI_FIRST) {
-    Util::ArabicToKanji(arabic_content_value, output);
+    NumberUtil::ArabicToWideArabic(arabic_content_value, output);
+    NumberUtil::ArabicToSeparatedArabic(arabic_content_value, output);
+    NumberUtil::ArabicToKanji(arabic_content_value, output);
+    NumberUtil::ArabicToOtherForms(arabic_content_value, output);
+  } else if (type == KANJI_FIRST) {
+    NumberUtil::ArabicToKanji(arabic_content_value, output);
     InsertHalfArabic(arabic_content_value, output);
-    Util::ArabicToWideArabic(arabic_content_value, output);
-    Util::ArabicToSeparatedArabic(arabic_content_value, output);
-    Util::ArabicToOtherForms(arabic_content_value, output);
+    NumberUtil::ArabicToWideArabic(arabic_content_value, output);
+    NumberUtil::ArabicToSeparatedArabic(arabic_content_value, output);
+    NumberUtil::ArabicToOtherForms(arabic_content_value, output);
   }
 
   if (exec_radix_conversion) {
-    Util::ArabicToOtherRadixes(arabic_content_value, output);
+    NumberUtil::ArabicToOtherRadixes(arabic_content_value, output);
   }
 }
 
-}  // namespace
-
-NumberRewriter::NumberRewriter(const POSMatcher *pos_matcher)
-    : pos_matcher_(pos_matcher) {}
-
-NumberRewriter::~NumberRewriter() {}
-
-int NumberRewriter::capability() const {
-  if (GET_REQUEST(mixed_conversion)) {
-    return RewriterInterface::ALL;
-  }
-  return RewriterInterface::CONVERSION;
-}
-
-struct RewriteCandidateInfo {
-  NumberRewriter::RewriteType type;
-  int position;
-  Segment::Candidate candidate;
-};
-
-bool NumberRewriter::RewriteOneSegment(bool exec_radix_conversion, Segment *seg)
-    const {
+bool RewriteOneSegment(
+    const POSMatcher &pos_matcher, bool exec_radix_conversion, Segment *seg) {
   DCHECK(seg);
   bool modified = false;
   vector<RewriteCandidateInfo> rewrite_candidate_infos;
-  GetRewriteCandidateInfos(*seg, &rewrite_candidate_infos);
+  GetRewriteCandidateInfos(*seg, pos_matcher, &rewrite_candidate_infos);
 
   for (int i = rewrite_candidate_infos.size() - 1; i >= 0; --i) {
     const RewriteCandidateInfo &info = rewrite_candidate_infos[i];
@@ -310,7 +383,7 @@ bool NumberRewriter::RewriteOneSegment(bool exec_radix_conversion, Segment *seg)
                  << arabic_content_value;
       break;
     }
-    vector<Util::NumberString> output;
+    vector<NumberUtil::NumberString> output;
     GetNumbers(info.type, exec_radix_conversion, arabic_content_value, &output);
     vector<Segment::Candidate> converted_numbers;
     for (int j = 0; j < output.size(); ++j) {
@@ -318,18 +391,33 @@ bool NumberRewriter::RewriteOneSegment(bool exec_radix_conversion, Segment *seg)
                         output[j].style, &converted_numbers);
     }
     SetCandidatesInfo(info.candidate, &converted_numbers);
-    int base_candidate_pos = info.position;
-    int insert_pos = GetInsertPos(base_candidate_pos, *seg, info.type);
-    EraseExistingCandidates(
-        converted_numbers, &base_candidate_pos, &insert_pos, seg);
-    DCHECK_LT(base_candidate_pos, insert_pos);
-    InsertConvertedCandidates(converted_numbers, info.candidate,
-                              base_candidate_pos,
-                              insert_pos, seg);
+
+    // Caution!!!: This invocation will update the data inside of the
+    // rewrite_candidate_infos. Thus, |info| also can be updated as well
+    // regardless of whether it's const reference-ness.
+    EraseExistingCandidates(converted_numbers, info.position,
+                            seg, &rewrite_candidate_infos);
+    int insert_pos = GetInsertPos(info.position, *seg, info.type);
+    DCHECK_LT(info.position, insert_pos);
+    InsertConvertedCandidates(
+        converted_numbers, info.candidate, info.position, insert_pos, seg);
     modified = true;
   }
   return modified;
 }
+}  // namespace
+
+NumberRewriter::NumberRewriter(const POSMatcher *pos_matcher)
+    : pos_matcher_(pos_matcher) {}
+
+NumberRewriter::~NumberRewriter() {}
+
+int NumberRewriter::capability() const {
+  if (GET_REQUEST(mixed_conversion)) {
+    return RewriterInterface::ALL;
+  }
+  return RewriterInterface::CONVERSION;
+}
 
 bool NumberRewriter::Rewrite(const ConversionRequest &request,
                              Segments *segments) const {
@@ -343,90 +431,15 @@ bool NumberRewriter::Rewrite(const ConversionRequest &request,
   // Radix conversion is done only for conversion mode.
   // Showing radix candidates is annoying for an user.
   const bool exec_radix_conversion =
-      (segments->conversion_segments_size() == 1
-       && segments->request_type() == Segments::CONVERSION);
+      (segments->conversion_segments_size() == 1 &&
+       segments->request_type() == Segments::CONVERSION);
 
   for (size_t i = 0; i < segments->conversion_segments_size(); ++i) {
     Segment *seg = segments->mutable_conversion_segment(i);
-    modified |= RewriteOneSegment(exec_radix_conversion, seg);
+    modified |= RewriteOneSegment(*pos_matcher_, exec_radix_conversion, seg);
   }
 
   return modified;
 }
 
-bool NumberRewriter::IsNumber(uint16 lid) const {
-  // Number candidates sometimes categorized as general noun.
-  // TODO(toshiyuki): It's better if we can rewrite
-  // from general noun POS to number POS
-  // TODO(toshiyuki): We can remove general noun check if we can set
-  // correct POS.
-  return (pos_matcher_->IsNumber(lid) || pos_matcher_->IsKanjiNumber(lid) ||
-          pos_matcher_->IsGeneralNoun(lid));
-}
-
-void NumberRewriter::GetRewriteCandidateInfos(
-    const Segment &seg,
-    vector<RewriteCandidateInfo> *rewrite_candidate_info) const {
-  DCHECK(rewrite_candidate_info);
-  RewriteCandidateInfo info;
-
-  for (size_t i = 0; i < seg.candidates_size(); ++i) {
-    const RewriteType type = GetRewriteTypeAndBase(
-        seg, i, &info.candidate);
-    if (type == NO_REWRITE) {
-      continue;
-    }
-    info.type = type;
-    info.position = i;
-    rewrite_candidate_info->push_back(info);
-  }
-}
-
-// Returns rewrite type for the given segment and base candidate information.
-// base_candidate_pos: the index of the base candidate.
-// *arabic_candidate: arabic candidate using numeric style conversion.
-// POS information, cost, etc will be copied from base candidate.
-NumberRewriter::RewriteType NumberRewriter::GetRewriteTypeAndBase(
-    const Segment &seg,
-    int base_candidate_pos,
-    Segment::Candidate *arabic_candidate) const {
-  DCHECK(arabic_candidate);
-
-  const Segment::Candidate &c = seg.candidate(base_candidate_pos);
-  if (!IsNumber(c.lid)) {
-    return NO_REWRITE;
-  }
-
-  if (Util::GetScriptType(c.content_value) == Util::NUMBER) {
-    arabic_candidate->CopyFrom(c);
-    return ARABIC_FIRST;
-  }
-
-  string half_width_new_content_value;
-  Util::FullWidthToHalfWidth(c.content_key, &half_width_new_content_value);
-  // Try to get normalized kanji_number and arabic_number.
-  // If it failed, do nothing.
-  // Retain suffix for later use.
-  string number_suffix, kanji_number, arabic_number;
-  if (!Util::NormalizeNumbersWithSuffix(c.content_value,
-                                        true,  // trim_reading_zeros
-                                        &kanji_number,
-                                        &arabic_number,
-                                        &number_suffix) ||
-      arabic_number == half_width_new_content_value) {
-    return NO_REWRITE;
-  }
-  const string suffix = c.value.substr(
-      c.content_value.size(), c.value.size() - c.content_value.size());
-  arabic_candidate->Init();
-  arabic_candidate->value = arabic_number + number_suffix + suffix;
-  arabic_candidate->content_value = arabic_number + number_suffix;
-  arabic_candidate->key = c.key;
-  arabic_candidate->content_key = c.content_key;
-  arabic_candidate->cost = c.cost;
-  arabic_candidate->structure_cost = c.structure_cost;
-  arabic_candidate->lid = c.lid;
-  arabic_candidate->rid = c.rid;
-  return KANJI_FIRST;
-}
 }  // namespace mozc
