@@ -31,6 +31,8 @@
 
 #include <string>
 
+#include "base/base.h"
+#include "base/logging.h"
 #include "base/singleton.h"
 #include "base/util.h"
 #include "config/config.pb.h"
