@@ -28,9 +28,14 @@
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <iostream>
+#include <string>
+
 #include "base/base.h"
 #include "base/file_stream.h"
+#include "base/logging.h"
 #include "base/util.h"
+#include "engine/engine_factory.h"
+#include "engine/engine_interface.h"
 #include "session/commands.pb.h"
 #include "session/key_parser.h"
 #include "session/session.h"
@@ -41,7 +46,8 @@ DEFINE_string(profile_dir, "", "Profile dir");
 
 namespace mozc {
 void Loop(istream *input, ostream *output) {
-  scoped_ptr<session::Session> session(new session::Session);
+  scoped_ptr<EngineInterface> engine(EngineFactory::Create());
+  scoped_ptr<session::Session> session(new session::Session(engine.get()));
 
   commands::Command command;
   string line;
@@ -51,7 +57,7 @@ void Loop(istream *input, ostream *output) {
       continue;
     }
     if (line.empty()) {
-      session.reset(new session::Session);
+      session.reset(new session::Session(engine.get()));
       *output << endl << "## New session" << endl << endl;
       continue;
     }
