@@ -27,6 +27,10 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+#ifdef OS_WINDOWS
+#include <Windows.h>
+#endif  // OS_WINDOWS
+
 #include <string>
 #include "base/base.h"
 #include "base/file_stream.h"
