@@ -32,11 +32,12 @@
 #endif  // OS_WINDOWS
 
 #include "base/base.h"
+#include "base/crash_report_util.h"
 #include "base/util.h"
-
 #ifdef OS_WINDOWS
 #include "base/winmain.h"
 #endif  // OS_WINDOWS
+#include "config/stats_config_util.h"
 
 DEFINE_string(mode, "", "mozc_broker mode");
 
@@ -57,7 +58,10 @@ int main(int argc, char *argv[]) {
 
   mozc::Util::DisableIME();
 
-  InitGoogleWithBreakPad(argv[0], &argc, &argv, false);
+  if (mozc::config::StatsConfigUtil::IsEnabled()) {
+    mozc::CrashReportUtil::InstallBreakpad();
+  }
+  InitGoogle(argv[0], &argc, &argv, false);
 
   int result = 0;
 #ifdef OS_WINDOWS
