@@ -95,6 +95,14 @@ class LanguageBar {
   HRESULT SetLangbarMenuEnabled(bool enable);
 
  private:
+  // Represents the language bar item manager iff the running OS is Windows 8.
+  // NOTE: We must use the same instance of this class to initialize and
+  //     uninitialize LangBar items. Otherwise, you will see weird crashes
+  //     around refcount on Windows 8 release preview. b/6106437
+  // NOTE: Currently we cannot use the same logic for Windows 7 due to another
+  //     crash issue as filed as b/6641460.
+  ATL::CComPtr<ITfLangBarItemMgr> lang_bar_item_mgr_for_win8_;
+
   // Represents the button menu in the language bar.
   // NOTE: ImeToggleButtonMenu inherits ITfLangBarItemButton and ITfSource,
   // which inherit IUnknown. Because of this, using CComPtr<ImeIconButtonMenu>
