@@ -34,15 +34,12 @@
 
 #include "base/port.h"
 #include "base/scoped_ptr.h"
-#include "win32/ime/ime_keyboard.h"
 
 namespace mozc {
-namespace commands {
-class Output;
-}  // namespace commands
-
 namespace win32 {
 
+class VirtualKey;
+
 class SurrogatePairObserver {
  public:
   // Return code which represents the expected action of the IME DLL.
@@ -62,8 +59,8 @@ class SurrogatePairObserver {
   };
 
   struct ClientAction {
-    inline ClientAction(ClientActionType type, char32 ucs4)
-         : type(type), ucs4(ucs4) {}
+    ClientAction(ClientActionType type, char32 ucs4)
+        : type(type), ucs4(ucs4) {}
     const ClientActionType type;
     const char32 ucs4;
   };
